!******************************************************************************
!
! $Id$
!
!******************************************************************************
!==============================================================================
!
! THE DESIGN AND TEST CALCULATION OF THE BUTTERWORTH FILTER.
!
!==============================================================================
PROGRAM MAIN_BUTTER
IMPLICIT NONE
INTEGER,PARAMETER:: RK=MY_RKIND
CHARACTER(*),PARAMETER:: FLT_NAME='BUTTER'
INTEGER,     PARAMETER:: NF_ATTEN=11
CHARACTER(*),PARAMETER:: FNAME_ATTEN='ATTEN_BUTTER.DAT'
INTEGER,     PARAMETER:: NF_POLES=22
CHARACTER(*),PARAMETER:: FNAME_POLES='POLES_BUTTER.DAT'
INTEGER,     PARAMETER:: NF_TBL=33
CHARACTER(*),PARAMETER:: FNAME_TBL='TBL_BUTTER.DAT'
INTEGER,     PARAMETER:: NF_EXPND=44
CHARACTER(*),PARAMETER:: FNAME_EXPND='EXPND_BUTTER.DAT'
INTEGER,     PARAMETER:: NF_REDUCED=55
CHARACTER(*),PARAMETER:: FNAME_REDUCED='REDUCED_EXPND_BUTTER.DAT'
REAL(RK) MU, A_MAX, A_MIN ! THE 3 SHAPE PARAMETERS OF ATTENUATION.
REAL(RK) EPS     ! "EPS" IN THE FORMULA OF ATTENUATION.
REAL(RK) A_STOP  ! THE TRUE LOWER BOUND OF THE ATTENUATION IN THE STOPBAND.
INTEGER N_MIN, N ! THE MINIMUM VALUE OF DEGREE "N_MIN", AND DEGREE "N".
REAL(RK) TL, TH  ! THE LOWER AND UPPER ABSCISSAS FOR GRAPH.
COMPLEX(RK),ALLOCATABLE:: ZPOLE(:), ZCOEF(:) ! POLES AND THEIR COEFFS.
REAL(RK)               :: CINF ! COEFFICIENT OF THE CONSTANT TERM.
!------------------------------------------------------------------------------
  PRINT *, 'DEBUG: PROGRAM MAIN_BUTTER'
  PRINT *, 'DEBUG: RKIND=',RK
  !====================
  ! READ FROM THE INPUT, THE TRIPLET PARAMETERS OF THE SHAPE OF THE ATTENUATION.
  CALL READ_PARAMS (MU, A_MAX, A_MIN)
  !====================
  ! CALCULATE THE VALUE OF "N_MIN".
  CALL FILTER_SETUP_BUTTER (MU, A_MAX, A_MIN, EPS, N_MIN)
  !====================
  ! READ FROM THE INPUT, "N" THE DEGREE OF THE FILTER.
  CALL READ_DEG (N_MIN, N)
  !====================
  ! OUTPUT THE TRUE LOWERBOUND OF THE ATTENUATION IN THE STOPBAND.
  CALL FILTER_ASTOP_BUTTER (EPS, MU, N, A_STOP)
  WRITE(*,100) 10*LOG10(A_STOP)
  100 FORMAT('A_STOP = ', F8.3,'[dB]')
  !====================
  ! INPUT THE RANGE OF THE ABSCISSAS OF "T" FOR THE PLOT.
#if 1
  PRINT *, 'INPUT TL, TH:'
  READ *, TL, TH
#else
  TH=1+5*(MU-1); TL=1-5*(MU-1) ! ONLY IN THE NEIGHBOUR OF THE TRANSITION REGION.
#endif
  !====================
  ! PLOT THE VALUE OF THE ATTENUATION FUNCTION BY THE FORMULA.
  OPEN (NF_ATTEN,   FILE=FNAME_ATTEN,   FORM='FORMATTED')
  CALL PLOT_GRAPH_BUTTER (NF_ATTEN, N, EPS, TL, TH)
  !====================
  ! CALCULATE THE POLES AND THEIR COEFFICIENTS OF THE TRANSFER FUNCTION.
  ALLOCATE (ZPOLE(N), ZCOEF(N))
  CALL FILTER_POLES_COEFS_BUTTER (N, EPS, ZPOLE, ZCOEF, CINF)
  !------
  ! CHECK THE CALCULATED COMPLEX POLES ARE ACCURATE ZEROS
  ! OF THE ATTENUATION FUNCTION.
  CALL CHECK_POLE_BUTTER (N, EPS, ZPOLE)
  !====================
  ! PLOT THE ABSCISSAS OF THE POLES IN THE COMPLEX PLANE.
  OPEN (NF_POLES,   FILE=FNAME_POLES,   FORM='FORMATTED')
  CALL PLOT_POLES (NF_POLES, N, ZPOLE)
  !====================
  ! OUTPUT THE CALCULATED POLES AND THEIR COEFFICIENTS TO THE FILE.
  OPEN (NF_TBL,     FILE=FNAME_TBL,     FORM='FORMATTED')
  CALL OUTPUT_COMMENT (NF_TBL, FLT_NAME, MU, A_MAX, A_MIN, N, A_STOP)
  CALL OUTPUT_POLES_COEFS (NF_TBL, FLT_NAME, N, ZPOLE, ZCOEF, CINF)
  !====================
  ! CHECK IF THE FRACTIONAL EXPANSION IS ACCURATE (COMPARED WITH FORMULA).
  CALL CHECK_EXPANSION_BUTTER (N, EPS, ZPOLE, ZCOEF, CINF, TL, TH)
  !====================
  ! PLOT THE ATTENUATION FUNCTION USING THE CALCULATED FRACTIONAL EXPANSION.
  OPEN (NF_EXPND,   FILE=FNAME_EXPND,   FORM='FORMATTED')
  CALL PLOT_EXPANSION (NF_EXPND, N, ZPOLE, ZCOEF, CINF, TL, TH)
  !====================
#if 0
  ! PLOT THE ATTENUATION FUNCTION USING THE CALCULATED FRACTIONAL EXPANSION 
  ! WITH REDUCED NUMERICAL ACCURACIES OF POLES AND THEIR COEFFICIENTS.
  OPEN (NF_REDUCED, FILE=FNAME_REDUCED, FORM='FORMATTED')
  CALL PLOT_REDUCED_EXPANSION (NF_REDUCED, N, ZPOLE, ZCOEF, CINF, TL, TH)
#endif
  !====================
  DEALLOCATE (ZPOLE, ZCOEF)
END PROGRAM MAIN_BUTTER
!==============================================================================
!
! PLOT THE GRAPH OF THE BUTTERWORTH FILTER.
!
!==============================================================================
SUBROUTINE PLOT_GRAPH_BUTTER (NFILE, N, EPS, TL, TH)
IMPLICIT NONE
INTEGER,PARAMETER:: RK=MY_RKIND
INTEGER,INTENT(IN):: NFILE
INTEGER, INTENT(IN):: N   ! DEGREE OF THE FILTER.
REAL(RK),INTENT(IN):: EPS ! "EPS" IN THE FORMULA OF THE ATTENUATION.
REAL(RK),INTENT(IN):: TL, TH ! THE RANGE OF THE ABSCISSAS FOR PLOT.
!------------------------------------------------------------------------------
INTEGER,PARAMETER:: NP = 10240 ! NUMBER OF PLOTS IN GRAPH.
CHARACTER(*),PARAMETER:: STR_LABEL = '"BUTTER"'
REAL(RK) T, A
INTEGER J
!----------------------------------------------------------
  WRITE (NFILE,'(A,3X,A,I5)') STR_LABEL, 'N=', N
  DO  J = 1, NP
    T = (TL*(NP-J) + TH*(J-1)) / (NP-1)
    A = 1 + (EPS*T**N)**2
    WRITE (NFILE,100) T, 10*LOG10(A)
    100 FORMAT (F25.20, 3X, F20.15)
  ENDDO
END SUBROUTINE PLOT_GRAPH_BUTTER
!==============================================================================
!
! CHECK IF THE POLES OF THE TRANSFER FUNCTIONS ARE SOLVED ACCURATE.
!
!==============================================================================
SUBROUTINE CHECK_POLE_BUTTER (N, EPS, ZPOLE)
IMPLICIT NONE
INTEGER,PARAMETER:: RK=MY_RKIND
INTEGER, INTENT(IN):: N   ! THE DEGREE OF FILTER.
REAL(RK),INTENT(IN):: EPS ! "EPS" IN THE FORMULA OF ATTENUATION.
COMPLEX(RK),INTENT(IN):: ZPOLE(N) ! THE COMPLEX POLES.
!------------------------------------------------------------------------------
REAL(RK) MAX_RES
!----------------------------------------------------------
  ! CHECK WHETHER IF THE SOLVED POLES ARE ACCURATE ENOUGH.
  MAX_RES = MAXVAL(ABS(1+(EPS*ZPOLE(:)**N)**2))
  WRITE (*,100) MAX_RES
  100 FORMAT ('MAX RESIDUAL OF "A" AT THE SOLVED POLES=', ES8.2)
#if 0
  IF (MAX_RES > EPSILON(1.0_RK)*10000) STOP 'ERROR. TOO LARGE MAX_RES.'
#endif
END SUBROUTINE CHECK_POLE_BUTTER
!==============================================================================
!
! CHECK THE FRACTIONAL EXPANSION OF BUTTERWORTH FILTER.
!
!==============================================================================
SUBROUTINE CHECK_EXPANSION_BUTTER (N, EPS, ZPOLE, ZCOEF, CINF, TL, TH)
IMPLICIT NONE
INTEGER,PARAMETER:: RK=MY_RKIND
INTEGER, INTENT(IN):: N   ! DEGREE OF THE FILTER.
REAL(RK),INTENT(IN):: EPS ! "EPS" IN THE FORMULA OF ATTENUATION.
COMPLEX(RK),INTENT(IN):: ZPOLE(N) ! POLES WHOSE IMAGINARY PARTS ARE POSITIVES,
COMPLEX(RK),INTENT(IN):: ZCOEF(N) ! AND THEIR CORRESPONDING COEFFICIENTS.
REAL(RK),   INTENT(IN):: CINF     ! THE COEFFICIENT FOR THE CONSTANT TERM.
REAL(RK),INTENT(IN):: TL, TH ! THE INTERVAL TO CHECK THE FILTER.
!------------------------------------------------------------------------------
INTEGER, PARAMETER:: NP = 10240 ! NUMBER OF ABSCISSAS OF "T" TO CHECK.
REAL(RK) ERR, T, ATTEN, G, DIFF
INTEGER J
!----------------------------------------------------------
  ERR = 0.0_RK
  DO  J = 1, NP
    T = (TL*(NP-J) + TH*(J-1)) / (NP-1)
    ATTEN = 1 + (EPS*T**N)**2
    G = CINF + 2*SUM(REAL(ZCOEF(:N)/(T-ZPOLE(:N)),KIND=RK))
    DIFF = ABS(G-1/ATTEN)
    ERR = MAX(ERR, DIFF)
  ENDDO
  WRITE(*,100) ERR
  WRITE(*,200) 0.5*MAXVAL(ABS(ZPOLE(:N)/AIMAG(ZPOLE(:N))))
  WRITE(*,300) MINVAL(ABS(AIMAG(ZPOLE(:N))))
  WRITE(*,400) MAXVAL(ABS(ZCOEF(:N)))
  100 FORMAT('MAX ERROR OF FRACTIONAL EXPANSION IS:', ES8.2)
  200 FORMAT('MAX Q-VALUE IS:', ES8.2)
  300 FORMAT('MIN SEPARATION FROM REAL AXIS=', ES8.2)
  400 FORMAT('MAX MAGNITUDE OF COEF=', ES8.2)
END SUBROUTINE CHECK_EXPANSION_BUTTER
