!==============================================================================
!
! THE DESIGN AND TEST CALCULATION OF THE CHEBYSHEV FILTER.
!
!==============================================================================
PROGRAM MAIN_CHEB
IMPLICIT NONE
INTEGER,PARAMETER:: RK=MY_RKIND
CHARACTER(*),PARAMETER:: FLT_NAME='CHEB'
INTEGER,     PARAMETER:: NF_ATTEN=11
CHARACTER(*),PARAMETER:: FNAME_ATTEN='ATTEN_CHEB.DAT'
INTEGER,     PARAMETER:: NF_POLES=22
CHARACTER(*),PARAMETER:: FNAME_POLES='POLES_CHEB.DAT'
INTEGER,     PARAMETER:: NF_TBL=33
CHARACTER(*),PARAMETER:: FNAME_TBL='TBL_CHEB.DAT'
INTEGER,     PARAMETER:: NF_EXPND=44
CHARACTER(*),PARAMETER:: FNAME_EXPND='EXPND_CHEB.DAT'
INTEGER,     PARAMETER:: NF_REDUCED=55
CHARACTER(*),PARAMETER:: FNAME_REDUCED='REDUCED_EXPND_CHEB.DAT'
REAL(RK) TL, TH  ! THE LOWER AND UPPER ABSCISSAS FOR GRAPH.
REAL(RK) MU, A_MAX, A_MIN ! THE SHAPE PARAMETERS OF THE ATTENUATION.
REAL(RK) EPS    ! "EPS" IN THE FORMULA OF ATTENUATION.
REAL(RK) A_STOP ! THE TRUE LOWER BOUND OF ATTENUATION IN THE STOPBAND.
INTEGER N_MIN, N ! MINIMUM VALUE OF DEGREE "N_MIN", AND DEGREE OF FILTER "N".
COMPLEX(RK),ALLOCATABLE:: ZPOLE(:), ZCOEF(:) ! POLES AND THEIR COEFFICIENTS.
REAL(RK)               :: CINF               ! COEFFICIENT OF THE CONSTANT TERM.
!------------------------------------------------------------------------------
  PRINT *, 'DEBUG: PROGRAM MAIN_CHEB'
  PRINT *, 'DEBUG: RKIND=',RK
  !====================
  ! READ FROM THE INPUT, THE TRIPLET PARAMETERS OF THE SHAPE OF THE ATTENUATION.
  CALL READ_PARAMS (MU, A_MAX, A_MIN)
  !====================
  ! CALCULATE THE VALUE OF "N_MIN".
  CALL FILTER_SETUP_CHEB (MU, A_MAX, A_MIN, EPS, N_MIN)
  !====================
  ! READ FROM THE INPUT, THE VALUE OF "N" THE DEGREE OF THE FILTER.
  CALL READ_DEG (N_MIN, N)
  !====================
  ! OUTPUT THE TRUE LOWERBOUND OF THE ATTENUATION IN THE STOPBAND.
  CALL FILTER_ASTOP_CHEB (EPS, MU, N, A_STOP)
  WRITE(*,100) 10*LOG10(A_STOP)
  100 FORMAT('A_STOP = ', F8.3,'[dB]')
  !====================
  ! INPUT THE RANGE OF THE ABSCISSAS OF "T" FOR THE PLOT.
#if 1
  PRINT *, 'INPUT TL, TH:'
  READ *, TL, TH
#else
  TH=1+5*(MU-1); TL=1-5*(MU-1) ! ONLY IN THE NEIGHBOUR OF THE TRANSITION REGION.
#endif
  !====================
  ! PLOT THE VALUE OF THE ATTENUATION FUNCTION BY THE FORMULA.
  OPEN (NF_ATTEN,   FILE=FNAME_ATTEN,   FORM='FORMATTED')
  CALL PLOT_GRAPH_CHEB (NF_ATTEN, N, EPS, TL, TH)
  !====================
  ! CALCULATE THE POLES AND THEIR COEFFICIENTS OF THE TRANSFER FUNCTION.
  ALLOCATE (ZPOLE(N), ZCOEF(N))
  CALL FILTER_POLES_COEFS_CHEB (N, EPS, ZPOLE, ZCOEF, CINF)
  !------
  ! CHECK THE CALCULATED COMPLEX POLES ARE ACCURATE ZEROS 
  ! OF THE ATTENUATION FUNCTION.
  CALL CHECK_POLES_CHEB (N, EPS, ZPOLE)
  !====================
  ! PLOT THE ABSCISSAS OF THE POLES IN THE COMPLEX PLANE.
  OPEN (NF_POLES,   FILE=FNAME_POLES,   FORM='FORMATTED')
  CALL PLOT_POLES (NF_POLES, N, ZPOLE)
  !====================
  ! OUTPUT THE CALCULATED POLES AND THEIR COEFFICIENTS TO THE FILE.
  OPEN (NF_TBL,     FILE=FNAME_TBL,     FORM='FORMATTED')
  CALL OUTPUT_COMMENT (NF_TBL, FLT_NAME, MU, A_MAX, A_MIN, N, A_STOP)
  CALL OUTPUT_POLES_COEFS (NF_TBL, FLT_NAME, N, ZPOLE, ZCOEF, CINF)
  !------
  ! CHECK IF THE FRACTIONAL EXPANSION IS ACCURATE (COMPARED WITH FORMULA).
  CALL CHECK_EXPANSION_CHEB (N, EPS, ZPOLE, ZCOEF, CINF, TL, TH)
  !====================
  ! PLOT THE ATTENUATION FUNCTION USING THE CALCULATED FRACTIONAL EXPANSION.
  OPEN (NF_EXPND,   FILE=FNAME_EXPND,   FORM='FORMATTED')
  CALL PLOT_EXPANSION (NF_EXPND, N, ZPOLE, ZCOEF, CINF, TL, TH)
  !====================
#if 0
  ! PLOT THE ATTENUATION FUNCTION USING THE CALCULATED FRACTIONAL EXPANSION
  ! WITH REDUCED NUMERICAL ACCURACIES OF THE POLES AND THEIR COEFFICIENTS.
  OPEN (NF_REDUCED, FILE=FNAME_REDUCED, FORM='FORMATTED')
  ! PLOT THE FRACTIONAL EXPANSION RESULT WITH REDUCED ACCURACIES.
  CALL PLOT_REDUCED_EXPANSION (NF_REDUCED, N, ZPOLE, ZCOEF, CINF, TL, TH)
#endif
  !====================
  DEALLOCATE (ZPOLE, ZCOEF)
END PROGRAM MAIN_CHEB
!==============================================================================
!
! PLOT THE GRAPHS OF THE CHEBYSHEV FILTER.
!
!==============================================================================
SUBROUTINE PLOT_GRAPH_CHEB (NFILE, N, EPS, TL, TH)
IMPLICIT NONE
INTEGER,PARAMETER:: RK=MY_RKIND
INTEGER, INTENT(IN):: NFILE
INTEGER, INTENT(IN):: N      ! THE DEGREE OF THE FILTER.
REAL(RK),INTENT(IN):: EPS    ! THE PARAMETER "EPS".
REAL(RK),INTENT(IN):: TL, TH ! THE RANGE OF ABSISSAS FOR PLOT.
!------------------------------------------------------------------------------
REAL(RK),EXTERNAL:: CHEB
INTEGER,PARAMETER:: NP=10240 ! NUMBER OF PLOTTED ABSCISSAS IN GRAPH.
CHARACTER(*),PARAMETER:: STR_LABEL = '"CHEB"'
REAL(RK) T, ATTEN
INTEGER J
!----------------------------------------------------------
WRITE (NFILE,'(A,3X,A,I5)') STR_LABEL, 'N=', N
DO  J = 1, NP
  T = (TL*(NP-J) + TH*(J-1)) / (NP-1)
  ATTEN = 1 + (EPS * CHEB(N,T))**2
  WRITE (NFILE,100) T, 10*LOG10(ATTEN)
  100 FORMAT (F25.20, 3X, F20.15)
ENDDO
END SUBROUTINE PLOT_GRAPH_CHEB
!==============================================================================
!
! CHECK IF THE SOLVED POLES OF THE TRANSFER FUNCTIONS ARE ACCURATE.
!
!==============================================================================
SUBROUTINE CHECK_POLES_CHEB (N, EPS, ZPOLE)
IMPLICIT NONE
INTEGER,PARAMETER:: RK=MY_RKIND
INTEGER, INTENT(IN):: N   ! THE DEGREE OF FILTER.
REAL(RK),INTENT(IN):: EPS ! "EPS" IN THE FORMULA OF ATTENUATION.
COMPLEX(RK),INTENT(IN):: ZPOLE(N) ! COMPLEX POLES.
!------------------------------------------------------------------------------
COMPLEX(RK),EXTERNAL:: ZCHEB
REAL(RK) MAX_RES
INTEGER J
!----------------------------------------------------------
  ! CHECK IF THE SOLVED POLES ARE ACCURATE ENOUGH.
  MAX_RES = 0.0_RK
  DO  J = 1, N
    MAX_RES = MAX(MAX_RES, ABS(1+(EPS*ZCHEB(N,ZPOLE(J)))**2) )
  ENDDO
  WRITE (*,100) MAX_RES
  100 FORMAT ('MAX RESIDUAL OF "A" AT THE SOLVED POLES=', ES8.2)
#if 0
  IF (MAX_RES > EPSILON(1.0_RK)*10000) STOP 'ERROR. TOO LARGE MAX_RES.'
#endif
END SUBROUTINE CHECK_POLES_CHEB
!==============================================================================
!
! CHECK IF THE FRACTIONAL EXPANSION OF THE CHEBYSHEV FILTER IS ACCURATE.
!
!==============================================================================
SUBROUTINE CHECK_EXPANSION_CHEB (N, EPS, ZPOLE, ZCOEF, CINF, TL, TH)
IMPLICIT NONE
INTEGER,PARAMETER:: RK=MY_RKIND
INTEGER,    INTENT(IN):: N        ! THE DEGREE "N".
REAL(RK),   INTENT(IN):: EPS      ! THE PARAMETER "EPS".
COMPLEX(RK),INTENT(IN):: ZPOLE(N) ! POLES WITH POSITIVE IMAGINARY PART.
COMPLEX(RK),INTENT(IN):: ZCOEF(N) ! COEFFICIENTS OF THE CORRESPONDING POLES.
REAL(RK),   INTENT(IN):: CINF     ! VALUE IS ALWAYS ZERO.
REAL(RK),   INTENT(IN):: TL, TH   ! THE RANGE OF ABSCISSAS FOR PLOT.
!------------------------------------------------------------------------------
REAL(RK),EXTERNAL:: CHEB
INTEGER, PARAMETER:: NP=10240 ! NUMBER OF PLOTS IN GRAPH.
REAL(RK) ERR, T, ATTEN, G, DIFF
INTEGER J
!----------------------------------------------------------
  ERR = 0.0_RK
  DO  J = 1, NP
    T = (TL*(NP-J) + TH*(J-1)) / (NP-1)
    ATTEN = 1 + (EPS * CHEB(N,T))**2
    G = CINF + 2*SUM(REAL(ZCOEF(:N)/(T-ZPOLE(:N)), KIND=RK))
    DIFF = ABS(G-1/ATTEN)
    ERR = MAX(ERR, DIFF)
  ENDDO
  WRITE(*,100) ERR
  WRITE(*,200) 0.5*MAXVAL(ABS(ZPOLE(:N)/AIMAG(ZPOLE(:N))))
  WRITE(*,300) MINVAL(ABS(AIMAG(ZPOLE(:N))))
  WRITE(*,400) MAXVAL(ABS(ZCOEF(:N)))
  100 FORMAT('MAX ERROR OF FRACTIONAL EXPANSION IS:', ES8.2)
  200 FORMAT('MAX Q-VALUE IS:', ES8.2)
  300 FORMAT('MIN SEPARATION FROM REAL AXIS=', ES8.2)
  400 FORMAT('MAX MAGNITUDE OF COEF=', ES8.2)
END SUBROUTINE CHECK_EXPANSION_CHEB
