!******************************************************************************
!
! $Id$
!
!******************************************************************************
!==============================================================================
!
! THE DESIGN AND TEST CALCULATION OF THE ELLIPTIC FILTER.
!
!==============================================================================
PROGRAM MAIN_ELLPTC
USE MOD_ELLPTC, ONLY: FILTER_SETUP_ELLPTC, FILTER_SETUP_ELLPTC2, &
                      FILTER_POLES_COEFS_ELLPTC, SET_RN_POLES_ZEROS
IMPLICIT NONE
INTEGER,PARAMETER:: RK=MY_RKIND
CHARACTER(*),PARAMETER:: FLT_NAME='ELLPTC'
INTEGER,     PARAMETER:: NF_ATTEN=11
CHARACTER(*),PARAMETER:: FNAME_ATTEN='ATTEN_ELLPTC.DAT'
INTEGER,     PARAMETER:: NF_POLES=22
CHARACTER(*),PARAMETER:: FNAME_POLES='POLES_ELLPTC.DAT'
INTEGER,     PARAMETER:: NF_TBL=33
CHARACTER(*),PARAMETER:: FNAME_TBL='TBL_ELLPTC.DAT'
INTEGER,     PARAMETER:: NF_EXPND=44
CHARACTER(*),PARAMETER:: FNAME_EXPND='EXPND_ELLPTC.DAT'
INTEGER,     PARAMETER:: NF_REDUCED=55
CHARACTER(*),PARAMETER:: FNAME_REDUCED='REDUCED_EXPND_ELLPTC.DAT'
REAL(RK) MU, A_MAX, A_MIN ! SHAPE PARAMETERS OF THE ATTENUATION.
REAL(RK) EPS, L ! "EPS", "L" IN THE FORMULA OF THE ATTENUATION.
REAL(RK) A_STOP ! THE LOWER BOUND OF THE ATTENUATION IN THE STOPBAND.
REAL(RK) TH, TL ! THE INTERVAL OF ABSCISSA FOR PLOT.
COMPLEX(RK),ALLOCATABLE:: ZPOLE(:), ZCOEF(:) ! POLES AND THEIR COEFFICIENTS.
REAL(RK)               :: CINF ! COEFFICIENT OF THE CONSTANT TERM.
INTEGER N_MIN, N ! MINIMAL VALUE OF DEGREE "N_MIN", DEGREE OF THE FILTER "N".
!------------------------------------------------------------------------------
  PRINT *, 'DEBUG: MAIN_ELLPTC'
  PRINT *, 'DEBUG: RKIND=',RK
  !====================
  ! READ FROM THE INPUT, THE TRIPLET PARAMETERS OF THE SHAPE OF THE ATTENUATION.
  CALL READ_PARAMS (MU, A_MAX, A_MIN)
  !====================
  CALL FILTER_SETUP_ELLPTC (MU, A_MAX, A_MIN, EPS, N_MIN) ! TO MAKE N_MIN.
  !====================
  ! READ FROM THE INPUT, "N" THE DEGREE OF THE FILTER.
  CALL READ_DEG (N_MIN, N)
  !====================
  CALL FILTER_SETUP_ELLPTC2 (MU, N, L) ! CALCULATE THE VALUE OF "L".
  ! THIS ROUTINE "SET_RN_POLES_ZEROS" MUST BE CALLED 
  ! BEFORE THE CALL OF "CMPLX_ELLPTC_RN" OR "ELLPTC_RN".
  CALL SET_RN_POLES_ZEROS (N, MU) 
  !====================
  ! OUTPUT THE TRUE LOWERBOUND OF THE ATTENUATION IN THE STOPBAND.
  CALL FILTER_ASTOP_ELLPTC (EPS, L, A_STOP) ! CALCULATE THE VALUE OF "A_STOP".
  WRITE(*,100) 10*LOG10(A_STOP)
  100 FORMAT('A_STOP = ', F8.3,'[dB]')
  !====================
  ! INPUT THE RANGE OF THE ABSCISSAS OF "T" FOR THE PLOT.
#if 1
  PRINT *, 'INPUT TL, TH:'
  READ *, TL, TH
#else
  TH = 1+5*(MU-1); TL=1-5*(MU-1) ! ONLY THE NEIGHBOUR OF THE TRANSITION REGION.
#endif
  !====================
  ! PLOT THE VALUE OF THE ATTENUATION FUNCTION BY THE FORMULA.
  OPEN (NF_ATTEN,   FILE=FNAME_ATTEN,   FORM='FORMATTED')
  CALL PLOT_GRAPH_ELLPTC (NF_ATTEN, N, EPS, TL, TH)
  !====================
  ! CALCULATE THE POLES AND THEIR COEFFICIENTS OF THE TRANSFER FUNCTION.
  ALLOCATE (ZPOLE(N), ZCOEF(N))
  CALL FILTER_POLES_COEFS_ELLPTC (N, MU, EPS, L, ZPOLE, ZCOEF, CINF)
  !------
  ! CHECK THE CALCULATED COMPLEX POLES ARE ACCURATE ZEROS
  ! OF THE ATTENUATION FUNCTION.
  CALL CHECK_POLES_ELLPTC (N,  EPS, ZPOLE)
  !====================
  ! PLOT THE ABSCISSAS OF THE POLES IN THE COMPLEX PLANE.
  OPEN (NF_POLES,   FILE=FNAME_POLES,   FORM='FORMATTED')
  CALL PLOT_POLES (NF_POLES, N, ZPOLE)
  !====================
  ! OUTPUT THE CALCULATED POLES AND THEIR COEFFICIENTS TO THE FILE.
  OPEN (NF_TBL,     FILE=FNAME_TBL,     FORM='FORMATTED')
  CALL OUTPUT_COMMENT (NF_TBL, FLT_NAME, MU, A_MAX, A_MIN, N, A_STOP)
  CALL OUTPUT_POLES_COEFS (NF_TBL, FLT_NAME, N, ZPOLE, ZCOEF, CINF)
  !====================
  ! CHECK IF THE FRACTIONAL EXPANSION IS ACCURATE (COMPARED WITH THE FORMULA).
  CALL CHECK_EXPANSION_ELLPTC (N, EPS, L, ZPOLE, ZCOEF, CINF, TL, TH)
  !====================
  ! PLOT THE ATTENUATION FUNCTION USING THE CALCULATED FRACTIONAL EXPANSION.
  OPEN (NF_EXPND,   FILE=FNAME_EXPND,   FORM='FORMATTED')
  CALL PLOT_EXPANSION (NF_EXPND, N, ZPOLE, ZCOEF, CINF, TL, TH)
  !====================
#if 0
  ! PLOT THE ATTENUATION FUNCTION USING THE CALCULATED FRACTIONAL EXPANSION
  ! WITH REDUCED NUMERICAL ACCURACIES OF POLES AND THEIR COEFFICIENTS.
  OPEN (NF_REDUCED, FILE=FNAME_REDUCED, FORM='FORMATTED')
  CALL PLOT_REDUCED_EXPANSION (NF_REDUCED, N, ZPOLE, ZCOEF, CINF, TL, TH)
#endif
  !====================
  DEALLOCATE (ZPOLE, ZCOEF)
END PROGRAM MAIN_ELLPTC
!==============================================================================
!
! PLOT THE ATTENUATION FUNCTION OF THE ELLIPTIC FILTER.
!
!==============================================================================
SUBROUTINE PLOT_GRAPH_ELLPTC (NFILE, N, EPS, TL, TH)
USE MOD_ELLPTC, ONLY: ELLPTC_RN
IMPLICIT NONE
INTEGER,PARAMETER:: RK=MY_RKIND
INTEGER, INTENT(IN):: NFILE
INTEGER, INTENT(IN):: N   ! THE DEGREE OF FILTER.
REAL(RK),INTENT(IN):: EPS ! "EPS" IN THE FORMULA OF ATTENUATION.
REAL(RK),INTENT(IN):: TL, TH ! INTERVAL OF ABSCISSA FOR PLOT.
!------------------------------------------------------------------------------
INTEGER,PARAMETER:: NP=10240 ! NUMBER OF PLOTS IN GRAPH.
CHARACTER(*),PARAMETER:: STR_LABEL = '"ELLPTC"'
REAL(RK) T, ATTEN
INTEGER J
!----------------------------------------------------------
  WRITE (NFILE,'(A,3X,A,I5)') STR_LABEL, 'N=', N
  DO  J = 1, NP
    T = (TL*(NP-J)+TH*(J-1)) / (NP-1)
    ATTEN = 1 + (EPS * ELLPTC_RN(N, T))**2
    WRITE (NFILE, 100) T, 10*LOG10(ATTEN)
    100 FORMAT (F25.20, 3X, F20.15)
  ENDDO
END SUBROUTINE PLOT_GRAPH_ELLPTC
!==============================================================================
!
! CHECK IF THE SOLVED POLES OF THE TRANSFER FUNCTION ARE ACCURATE.
!
!==============================================================================
SUBROUTINE CHECK_POLES_ELLPTC (N,  EPS, ZPOLE)
USE MOD_ELLPTC, ONLY: CMPLX_ELLPTC_RN
IMPLICIT NONE
INTEGER,PARAMETER:: RK=MY_RKIND
INTEGER, INTENT(IN):: N   ! THE DEGREE OF THE FILTER.
REAL(RK),INTENT(IN):: EPS ! "EPS" IN THE FORMULA OF ATTENUATION.
COMPLEX(RK),INTENT(IN):: ZPOLE(N) ! COMPLEX POLES.
!------------------------------------------------------------------------------
REAL(RK) MAX_RES, RES
INTEGER J
!----------------------------------------------------------
  ! CHECK IF THE SOLVED POLES ARE ACCURATE ENOUGH.
  MAX_RES = 0.0_RK
  DO  J = 1, N
    RES = ABS(1+(EPS*CMPLX_ELLPTC_RN(N, ZPOLE(J)))**2)
    MAX_RES = MAX(MAX_RES, RES)
  ENDDO
  WRITE (*,100) MAX_RES
  100 FORMAT ('MAX RESIDUAL OF "A" AT THE SOLVED POLES=', ES8.2)
#if 0
  IF (MAX_RES > EPSILON(1.0_RK)*10000) STOP 'ERROR. TOO LARGE RESIDUAL.'
#endif
END SUBROUTINE CHECK_POLES_ELLPTC
!==============================================================================
!
! CHECK IF THE FRACTIONAL EXPANSION IS ACCURATE.
!
!==============================================================================
SUBROUTINE CHECK_EXPANSION_ELLPTC (N, EPS, L, ZPOLE, ZCOEF, CINF, TL, TH)
USE MOD_ELLPTC,ONLY: ELLPTC_RN
IMPLICIT NONE
INTEGER,PARAMETER:: RK=MY_RKIND
INTEGER, INTENT(IN):: N ! DEGREE OF THE FILTER.
REAL(RK),INTENT(IN):: EPS, L ! "EPS", "L" IN THE FORMULA OF ATTENUATION.
COMPLEX(RK),INTENT(IN):: ZPOLE(N), ZCOEF(N) ! POLES AND THEIR COEFFICIENTS.
REAL(RK),   INTENT(IN):: CINF ! COEFFICIENT OF THE CONSTANT TERM.
REAL(RK),INTENT(IN):: TL, TH ! INTERVAL OF ABSCISSAS FOR THE CHECK.
!------------------------------------------------------------------------------
INTEGER,PARAMETER:: NP=10240 ! NUMBER POINTS FOR CHECK.
REAL(RK) ERR, T, G, ATTEN, DIFF
INTEGER J
!----------------------------------------------------------
  ERR = 0.0_RK
  DO  J = 1, NP
    T = (TL*(NP-J) + TH*(J-1)) / (NP-1)
    G = CINF + 2*SUM(REAL(ZCOEF(:N)/(T-ZPOLE(:N)),KIND=RK))
    ATTEN = 1 + (EPS * ELLPTC_RN(N, T))**2
    DIFF = ABS(G-1/ATTEN)
    ERR = MAX(ERR, DIFF)
  ENDDO
  WRITE(*,100) ERR
  WRITE(*,200) 0.5*MAXVAL(ABS(ZPOLE(:N)/AIMAG(ZPOLE(:N))))
  WRITE(*,300) MINVAL(ABS(AIMAG(ZPOLE(:N))))
  WRITE(*,400) MAXVAL(ABS(ZCOEF(:N)))
  100 FORMAT('MAX ERROR OF FRACTIONAL EXPANSION IS:', ES8.2)
  200 FORMAT('MAX Q-VALUE IS:', ES8.2)
  300 FORMAT('MIN SEPARATION FROM REAL AXIS=', ES8.2)
  400 FORMAT('MAX MAGNITUDE OF COEF=', ES8.2)
END SUBROUTINE CHECK_EXPANSION_ELLPTC
