!******************************************************************************
!
! $Id$
!
!******************************************************************************
!==============================================================================
!
! THE DESIGN AND TEST CALCULATION OF THE INVERSE CHEBYSHEV FILTER.
!
!==============================================================================
PROGRAM MAIN_INVCHEB
IMPLICIT NONE
INTEGER,PARAMETER:: RK=MY_RKIND
CHARACTER(*),PARAMETER:: FLT_NAME='INVCHEB'
INTEGER,     PARAMETER:: NF_ATTEN=11
CHARACTER(*),PARAMETER:: FNAME_ATTEN='ATTEN_INVCHEB.DAT'
INTEGER,     PARAMETER:: NF_POLES=22
CHARACTER(*),PARAMETER:: FNAME_POLES='POLES_INVCHEB.DAT'
INTEGER,     PARAMETER:: NF_TBL=33
CHARACTER(*),PARAMETER:: FNAME_TBL='TBL_INVCHEB.DAT'
INTEGER,     PARAMETER:: NF_EXPND=44
CHARACTER(*),PARAMETER:: FNAME_EXPND='EXPND_INVCHEB.DAT'
INTEGER,     PARAMETER:: NF_REDUCED=55
CHARACTER(*),PARAMETER:: FNAME_REDUCED='REDUCED_EXPND_INVCHEB.DAT'
REAL(RK) TL, TH  ! THE LOWER AND UPPER ABSCISSAS FOR GRAPH.
REAL(RK) MU, A_MAX, A_MIN ! SHAPE PARAMETERS FOR THE ATTENUATION.
REAL(RK) EPS ! "EPS" IN THE FORMULA OF THE ATTENUATION.
REAL(RK) A_STOP ! THE LOWER BOUND OF THE ATTENUATION IN THE STOPBAND.
INTEGER N_MIN, N ! THE MINIMAL DEGREE "N_MIN" AND THE DEGREE OF THE FILTER "N"
COMPLEX(RK),ALLOCATABLE:: ZPOLE(:), ZCOEF(:) ! POLES AND THEIR COEFFICIENTS.
REAL(RK)               :: CINF ! COEFFICIENT OF THE CONSTANT TERM.
!------------------------------------------------------------------------------
  PRINT *, 'DEBUG: MAIN_INVCHEB'
  PRINT *, 'DEBUG: RKIND=',RK
  !====================
  ! READ FROM THE INPUT, THE TRIPLET PARAMETERS OF THE SHAPE OF THE ATTENUATION.
  CALL READ_PARAMS (MU, A_MAX, A_MIN)
  !====================
  ! CALCULATE THE VALUE OF "N_MIN".
  CALL FILTER_SETUP_INVCHEB (MU, A_MAX, A_MIN, EPS, N_MIN)
  !====================
  ! READ FROM THE INPUT, THE VALUE OF "N" THE DEGREE OF THE FILTER.
  CALL READ_DEG (N_MIN, N)
  !====================
  ! OUTPUT THE TRUE LOWERBOUND OF THE ATTENUATION IN THE STOPBAND.
  CALL FILTER_ASTOP_INVCHEB (EPS, MU, N, A_STOP)
  WRITE(*,100) 10*LOG10(A_STOP)
  100 FORMAT('A_STOP = ', F8.3,'[dB]')
  !====================
  ! INPUT THE RANGE OF THE ABSCISSAS OF "T" FOR THE PLOT.
#if 1
  PRINT *, 'INPUT TL, TH:'
  READ *, TL, TH
#else
  TH=1+5*(MU-1); TL=1-5*(MU-1) ! ONLY IN THE NEIGHBOUR OF THE TRANSITION REGION.
#endif
  !====================
  ! PLOT THE VALUE OF THE ATTENUATION FUNCTION BY THE FORMULA.
  OPEN (NF_ATTEN,   FILE=FNAME_ATTEN,   FORM='FORMATTED')
  CALL PLOT_GRAPH_INVCHEB (NF_ATTEN, N, MU, EPS, TL, TH)
  !====================
  ! CALCULATE THE POLES AND THEIR COEFFICIENTS OF THE TRANSFER FUNCTION.
  ALLOCATE (ZPOLE(N), ZCOEF(N))
  CALL FILTER_POLES_COEFS_INVCHEB (N, MU, EPS, ZPOLE, ZCOEF, CINF)
  !------
  ! CHECK THE CALCULATED COMPLEX POLES ARE ACCURATE ZEROS 
  ! OF THE ATTENUATION FUNCTION.
  CALL CHECK_POLES_INVCHEB (N, MU, EPS, ZPOLE)
  !====================
  ! PLOT THE ABSCISSAS OF THE POLES IN THE COMPLEX PLANE.
  OPEN (NF_POLES,   FILE=FNAME_POLES,   FORM='FORMATTED')
  CALL PLOT_POLES (NF_POLES, N, ZPOLE)
  !====================
  ! OUTPUT THE CALCULATED POLES AND THEIR COEFFICIENTS TO THE FILE.
  OPEN (NF_TBL,     FILE=FNAME_TBL,     FORM='FORMATTED')
  CALL OUTPUT_COMMENT (NF_TBL, FLT_NAME, MU, A_MAX, A_MIN, N, A_STOP)
  CALL OUTPUT_POLES_COEFS (NF_TBL, FLT_NAME, N, ZPOLE, ZCOEF, CINF)
  !====================
  ! CHECK IF THE FRACTIONAL EXPANSION IS ACCURATE (COMPARED WITH FORMULA).
  CALL CHECK_EXPANSION_INVCHEB (N, MU, EPS, ZPOLE, ZCOEF, CINF, TL, TH)
  !====================
  ! PLOT THE ATTENUATION FUNCTION USING THE CALCULATED FRACTIONAL EXPANSION.
  OPEN (NF_EXPND,   FILE=FNAME_EXPND,   FORM='FORMATTED')
  CALL PLOT_EXPANSION (NF_EXPND, N, ZPOLE, ZCOEF, CINF, TL, TH)
  !====================
#if 0
  ! PLOT THE ATTENUATION FUNCTION USING THE CALCULATED FRACTIONAL EXPANSION.
  ! WITH REDUCED NUMERICAL ACCURACIES OF POLES AND THEIR COEFFICIENTS.
  OPEN (NF_REDUCED, FILE=FNAME_REDUCED, FORM='FORMATTED')
  CALL PLOT_REDUCED_EXPANSION (NF_REDUCED, N, ZPOLE, ZCOEF, CINF, TL, TH)
#endif
  !====================
  DEALLOCATE (ZPOLE, ZCOEF)
END PROGRAM MAIN_INVCHEB
!==============================================================================
!
! PLOT THE GRAPH OF THE ATTENUATION OF THE INVERSE-CHEBYSHEV FILTER.
!
!==============================================================================
SUBROUTINE PLOT_GRAPH_INVCHEB (NFILE, N, MU, EPS, TL, TH)
IMPLICIT NONE
INTEGER,PARAMETER:: RK=MY_RKIND
INTEGER, INTENT(IN):: NFILE
INTEGER, INTENT(IN):: N       ! THE DEGREE OF THE FILTER.
REAL(RK),INTENT(IN):: MU, EPS ! THE FILTER PARAMETERS.
REAL(RK),INTENT(IN):: TL, TH  ! THE RANGE OF ABSCISSAS FOR PLOT.
!------------------------------------------------------------------------------
REAL(RK),EXTERNAL:: CHEB
INTEGER,     PARAMETER:: NP=10240 ! NUMBER OF PLOTS IN GRAPH.
CHARACTER(*),PARAMETER:: STR_LABEL='"INVCHEB"'
REAL(RK) L, T, THETA, ATTEN
INTEGER J
!----------------------------------------------------------
  L = CHEB(N, MU)
  WRITE (NFILE,'(A,3X,A,I5)') STR_LABEL, "N=", N
  DO  J = 1, NP
    T = (TL*(NP-J)+TH*(J-1)) / (NP-1)
    IF (ABS(T) > EPSILON(T)) THEN
      IF (MU/T > 1.0_RK) THEN
        THETA = ACOSH(MU/T)
        ATTEN = 1 + (EPS * L / COSH(N*THETA))**2
      ELSE
        THETA = ACOS(MU/T)
        ATTEN = 1 + (EPS * L / COS(N*THETA))**2
      ENDIF
    ELSE
      IF (N == 0) THEN
        ATTEN = 1 + (EPS*L)**2
      ELSE
        ATTEN = 1
      ENDIF
    ENDIF
    WRITE (NFILE,100) T, 10*LOG10(ATTEN)
    100 FORMAT (F25.20, 3X, F20.15)
  ENDDO
END SUBROUTINE PLOT_GRAPH_INVCHEB
!==============================================================================
!
! CHECK IF THE SOLVED POLES OF THE TRANSFER FUNCTION ARE ACCURATE ENOUGH.
!
!==============================================================================
SUBROUTINE CHECK_POLES_INVCHEB (N, MU, EPS, ZPOLE)
IMPLICIT NONE
INTEGER,PARAMETER:: RK=MY_RKIND
INTEGER, INTENT(IN):: N ! DEGREE OF THE FILTER.
REAL(RK),INTENT(IN):: MU, EPS ! "MU", "EPS" IN THE FORMULA OF ATTENUATION.
COMPLEX(RK),INTENT(IN):: ZPOLE(N) ! COMPLEX POLES.
!------------------------------------------------------------------------------
REAL(RK),   EXTERNAL:: CHEB
COMPLEX(RK),EXTERNAL:: ZCHEB
REAL(RK) L, MAX_RES, RES
INTEGER J
!----------------------------------------------------------
  L = CHEB(N, MU)
  MAX_RES = 0.0_RK
  DO  J = 1, N
    RES = ABS(1+(EPS*L/ZCHEB(N,MU/ZPOLE(J)))**2)
    MAX_RES = MAX(MAX_RES, RES)
  ENDDO
  WRITE (*,100) MAX_RES
  100 FORMAT ('MAX RESIDUAL OF "A" AT THE SOLVED POLES=', ES8.2)
#if 0
  IF (MAX_RES > EPSILON(1.0_RK)*10000) STOP 'ERROR. TOO LARGE MAX_RES.'
#endif
END SUBROUTINE CHECK_POLES_INVCHEB
!==============================================================================
!
! CHECK THE FRACTIONAL EXPANSION OF INVERSE-CHEBYSHEV FILTER.
!
!==============================================================================
SUBROUTINE CHECK_EXPANSION_INVCHEB (N, MU, EPS, ZPOLE, ZCOEF, CINF, TL, TH)
IMPLICIT NONE
INTEGER,PARAMETER:: RK= MY_RKIND
INTEGER,    INTENT(IN):: N        ! THE DEGREE "N".
REAL(RK),   INTENT(IN):: EPS, MU  ! THE FILTER PARAMETERS.
COMPLEX(RK),INTENT(IN):: ZPOLE(N) ! POLES WITH POSITIVE IMAGINARY PARTS.
COMPLEX(RK),INTENT(IN):: ZCOEF(N) ! COEFFICIENTS OF THE CORRESPONDING POLES.
REAL(RK),   INTENT(IN):: CINF     ! COEFFICIENT OF THE CONSTANT TERM.
REAL(RK),   INTENT(IN):: TL, TH   ! THE RANGE OF ABSCISSAS FOR PLOT.
!------------------------------------------------------------------------------
REAL(RK),EXTERNAL:: CHEB
INTEGER,PARAMETER:: NP=10240 ! NUMBER OF PLOTS IN GRAPH.
REAL(RK) L, ERR, T, ATTEN, G, DIFF
INTEGER J
!----------------------------------------------------------
  L = CHEB(N, MU)
  !---
  ERR = 0.0_RK
  DO  J = 1, NP
    T = (TL*(NP-J) + TH*(J-1)) / (NP-1)
    IF (T /= 0.0) THEN
      ATTEN = 1 + (EPS*L / CHEB(N, MU/T))**2
    ELSE
      IF (N == 0) THEN
        ATTEN = 1 + EPS**2
      ELSE
        ATTEN = 1
      ENDIF
    ENDIF
    G = CINF + 2*SUM(REAL(ZCOEF(:N)/(T-ZPOLE(:N)), KIND=RK))
    DIFF = ABS(G-1/ATTEN)
    ERR = MAX(ERR, DIFF)
  ENDDO
  WRITE(*,100) ERR
  WRITE(*,200) 0.5*MAXVAL(ABS(ZPOLE(:N)/AIMAG(ZPOLE(:N))))
  WRITE(*,300) MINVAL(ABS(AIMAG(ZPOLE(:N))))
  WRITE(*,400) MAXVAL(ABS(ZCOEF(:N)))
  100 FORMAT('MAX ERROR OF FRACTIONAL EXPANSION IS:', ES8.2)
  200 FORMAT('MAX Q-VALUE IS:', ES8.2)
  300 FORMAT('MIN SEPARATION FROM REAL AXIS=', ES8.2)
  400 FORMAT('MAX MAGNITUDE OF COEF=', ES8.2)
END SUBROUTINE CHECK_EXPANSION_INVCHEB
