!===============================================================================
!
!===============================================================================
SUBROUTINE READ_PARAMS (MU, A_MAX, A_MIN)
IMPLICIT NONE
INTEGER,PARAMETER:: RK=MY_RKIND
REAL(RK),INTENT(OUT):: MU, A_MAX, A_MIN ! SHAPE PARAMETERS OF ATTENUATION.
!---------------------------------
  WRITE (*,*) 'INPUT MU:'
  READ *, MU
  WRITE (*,*) 'MU=', MU
  !--
  WRITE (*,*) 'INPUT A_MAX[dB], A_MIN[dB]:'
  READ *, A_MAX, A_MIN
  WRITE (*,*) 'A_MAX, A_MIN=', A_MAX, A_MIN
  A_MAX = (10.0_RK)**(A_MAX/10.0_RK)
  A_MIN = (10.0_RK)**(A_MIN/10.0_RK)
  !---
END SUBROUTINE READ_PARAMS
!===============================================================================
!
!===============================================================================
SUBROUTINE READ_DEG (N_MIN, N)
IMPLICIT NONE
INTEGER,INTENT(IN):: N_MIN ! MINIMAL DEGREE TO MEET THE SHAPE PARAMETERS.
INTEGER,INTENT(OUT):: N    ! SPECIFIED VALUE OF DEGREE.
!----------------------------
  WRITE (*,*) 'N_MIN=', N_MIN
  WRITE (*,*) 'INPUT N (THE HALF DEGREE OF THE FILTER):'
  READ (*,*) N
  IF (N == 0) THEN
    N = N_MIN ! WHEN THE INPUT IS ZERO, THE MINIMAL VALUE IS SET TO N.
    WRITE (*,100) N
    100 FORMAT('N= ', I10, '(THE MINIMAL VALUE IS SET)')
  ELSEIF (N < 0) THEN
    N = ABS(N) ! WHEN THE INPUT IS NEGATIVE, ITS ABSOLUTE VALUE IS SET TO N.
    WRITE (*,110) N
    110 FORMAT('N IS FORCED TO: N= ', I10, '(FORCED BY THE INPUT)')
  ELSEIF (N < N_MIN) THEN
    WRITE (*,120) N
    120 FORMAT('N= ', I10, '(ERROR: N IS LESS THAN N_MIN)')
    STOP 'ERROR: N IS LESS THAN N_MIN'
  ELSE
    WRITE (*,130) N
    130 FORMAT('N= ', I10, '(VALUE IS SET)' )
  ENDIF
END SUBROUTINE READ_DEG
!===============================================================================
!
!===============================================================================
SUBROUTINE OUTPUT_COMMENT (NFILE, FLT_NAME, MU, A_MAX, A_MIN, N, A_STOP)
IMPLICIT NONE
INTEGER,PARAMETER:: RK=MY_RKIND
INTEGER,INTENT(IN):: NFILE
CHARACTER(*),INTENT(IN):: FLT_NAME
REAL(RK),INTENT(IN):: MU, A_MAX, A_MIN ! SHAPE PARAMETERS OF ATTENUATION.
INTEGER, INTENT(IN):: N ! DEGREE OF THE FILTER.
REAL(RK),INTENT(IN):: A_STOP ! TRUE LOWER BOUND OF ATTENUATION IN STOPBAND.
!--------------------------------------
  WRITE(NFILE,'(A)') '! POLES AND COEFFICIENTS (FILTER='//FLT_NAME//').'
  WRITE(NFILE, 100) MU, 10*LOG10(A_MAX), 10*LOG10(A_MIN)
  100 FORMAT('! MU=',F12.8,', A_MAX=', F10.3,'[dB], A_MIN=', F10.3,'[dB]')
  WRITE(NFILE,'(A,I10)') '! N=', N
  WRITE(NFILE,200) 10*LOG10(A_STOP)
  200 FORMAT('! A_STOP = ', F8.3,'[dB]')
END SUBROUTINE OUTPUT_COMMENT
!===============================================================================
!
!===============================================================================
SUBROUTINE OUTPUT_POLES_COEFS (NFILE, FLT_NAME, N, ZPOLE, ZCOEF, CINF)
IMPLICIT NONE
INTEGER,PARAMETER:: RK=MY_RKIND
INTEGER,     INTENT(IN):: NFILE
CHARACTER(*),INTENT(IN):: FLT_NAME
INTEGER,     INTENT(IN):: N  ! DEGREE OF FILTER.
COMPLEX(RK), INTENT(IN):: ZPOLE(N), ZCOEF(N) ! POLES AND THEIR COEFFICIENTS.
REAL(RK),    INTENT(IN):: CINF ! COEFFICIENT OF THE CONSTANT TERM.
!----------------------------------------------------
CHARACTER(80) FMT_CINF, FMT_POLE, FMT_COEF, FMT_LABEL
CHARACTER CH
INTEGER J
!-------------
  SELECT CASE(RK)
  CASE(KIND(1.E0))
    FMT_CINF = "(2X,'CINF=',E15.8)"
    FMT_POLE = "(2X,'ZPOLE(',I9,')=(',E15.8,',', E15.8,')')"
    FMT_COEF = "(2X,'ZCOEF(',I9,')=(',E15.8,',', E15.8,')')"
  CASE(KIND(1.D0))
    FMT_CINF = "(2X,'CINF=',D23.16)"
    FMT_POLE = "(2X,'ZPOLE(',I9,')=(',D23.16,',', D23.16,')')"
    FMT_COEF = "(2X,'ZCOEF(',I9,')=(',D23.16,',', D23.16,')')"
#ifndef NO_QUADRUPLE_PRECISION
  CASE(KIND(1.Q0))
    FMT_CINF = "(2X,'CINF=',E42.35,'_RK')"
    FMT_POLE = "(2X,'ZPOLE(',I9,')=(',E42.35,'_RK,', E42.35,'_RK)')"
    FMT_COEF = "(2X,'ZCOEF(',I9,')=(',E42.35,'_RK,', E42.35,'_RK)')"
#endif
  END SELECT
  FMT_LABEL="(A,I9:A)"
  SELECT CASE(N)
    CASE(1:9)        ; CH='1'
    CASE(10:99)      ; CH='2'
    CASE(100:999)    ; CH='3'
    CASE(1000:9999)  ; CH='4'
    CASE(10000:99999); CH='5'
  END SELECT
  FMT_POLE(15:15)=CH
  FMT_COEF(15:15)=CH
  FMT_LABEL(5:5)=CH
  !---
  WRITE(NFILE,FMT_LABEL)'SUBROUTINE SET_'//FLT_NAME//'_POLES_COEFS_',N, &
                        '(ZPOLE, ZCOEF, CINF)'
  WRITE(NFILE,"(A)")'IMPLICIT NONE'
  WRITE(NFILE,FMT_LABEL)'INTEGER,PARAMETER:: N=', N
  WRITE(NFILE,"(A)")'COMPLEX(KIND=RK),INTENT(OUT):: ZPOLE(N), ZCOEF(N)'
  WRITE(NFILE,"(A)")'REAL(KIND=RK),   INTENT(OUT):: CINF'
  WRITE(NFILE,FMT_CINF) CINF
  DO  J = 1, N
    WRITE(NFILE,FMT_POLE) J, ZPOLE(J)
    WRITE(NFILE,FMT_COEF) J, ZCOEF(J)
  ENDDO
  WRITE(NFILE,FMT_LABEL) 'END SUBROUTINE SET_'//FLT_NAME//'_POLES_COEFS_',N
END SUBROUTINE OUTPUT_POLES_COEFS
!===============================================================================
! REDUCE THE ACCURACY OF THE COMPLEX NUMBER 
! TO THE SPECIFIED DIGITS.
!===============================================================================
SUBROUTINE Z_REDUCE_ACCURACY (Z, DIGITS)
IMPLICIT NONE
INTEGER,PARAMETER:: RK=MY_RKIND
COMPLEX(RK),INTENT(INOUT):: Z ! A REAL NUMBER TO REDUCE ITS ACCURACY.
INTEGER,    INTENT(IN)   :: DIGITS ! THE ACCURACY SPECIFIED IN DIGITS.
!----------------------------------------
REAL(RK) X, Y
CHARACTER(80) STR, FMT
!---------------------
  SELECT CASE (DIGITS)
    CASE (1:9)
      WRITE(FMT,100) DIGITS
      100 FORMAT("(E40.",I1,")")
    CASE (10:99)
      WRITE(FMT,200) DIGITS
      200 FORMAT("(E40.",I2,")")
    CASE DEFAULT
      STOP 'Z_REDUCE_ACCURACY: DIGITS OUT OF RANGE.'
  END SELECT
  WRITE(STR,FMT) REAL(Z,KIND=RK)
  READ (STR,FMT) X
  WRITE(STR,FMT) AIMAG(Z)
  READ (STR,FMT) Y
  Z = CMPLX(X, Y, KIND=RK)
END SUBROUTINE Z_REDUCE_ACCURACY
!===============================================================================
! PLOT THE POLES.
!===============================================================================
SUBROUTINE PLOT_POLES (NFILE, N, ZPOLE)
IMPLICIT NONE
INTEGER,PARAMETER:: RK=MY_RKIND
INTEGER,INTENT(IN):: NFILE
INTEGER,    INTENT(IN):: N        ! DEGREE OF THE FILTER.
COMPLEX(RK),INTENT(IN):: ZPOLE(N) ! POLES WHOSE IMAGINARY PARTS ARE POSITIVES.
!--------------------------------------
INTEGER J
!--------
  WRITE (NFILE,100)'# N=', N
  100 FORMAT (A, I8)
  DO  J = 1, N
    WRITE (NFILE,200) ZPOLE(J)
    200 FORMAT (F20.15, 3X, F20.15)
  ENDDO
  DO  J = N, 1, -1
    WRITE (NFILE,200) CONJG(ZPOLE(J))
  ENDDO
END SUBROUTINE PLOT_POLES
!===============================================================================
! PLOT THE FRACTIONAL EXPANSION OF THE FILTER TRANSFER FUNCTION.
!===============================================================================
SUBROUTINE PLOT_EXPANSION (NFILE, N, ZPOLE, ZCOEF, CINF, TL, TH)
IMPLICIT NONE
INTEGER,PARAMETER:: RK=MY_RKIND
INTEGER,    INTENT(IN):: NFILE
INTEGER,    INTENT(IN):: N        ! DEGREE OF THE FILTER.
COMPLEX(RK),INTENT(IN):: ZPOLE(N) ! POLES WITH POSITIVE IMAGINARY PARTS.
COMPLEX(RK),INTENT(IN):: ZCOEF(N) ! COEFFICIENTS OF CORRESPONDING POLES.
REAL(RK),   INTENT(IN):: CINF     ! COEFFICIENT OF THE CONSTANT TERM.
REAL(RK),   INTENT(IN):: TL, TH ! THE RANGE OF THE ABSCISSAS FOR PLOT.
!----------------------------------------
INTEGER,PARAMETER:: NP=10240 ! NUMBER POINTS FOR CHECK.
REAL(RK) T, G
INTEGER J
!----------------------------------------
  DO  J = 1, NP
    T = (TL*(NP-J) + TH*(J-1)) / (NP-1)
    G = CINF + 2 * SUM(REAL(ZCOEF(:N)/(T-ZPOLE(:N)),KIND=RK))
    IF (ABS(G) == 0.0_RK) CYCLE
    WRITE (NFILE,100) T, -10*LOG10(ABS(G))
    100 FORMAT (F25.20, 3X, F20.15)
  ENDDO
END SUBROUTINE PLOT_EXPANSION
!===============================================================================
! PLOT THE FRACTIONAL EXPANSION OF THE FILTER TRANSFER FUNCTION
! WITH REDUCED ACCURACIES.
!===============================================================================
SUBROUTINE PLOT_REDUCED_EXPANSION (NFILE, N, ZPOLE, ZCOEF, CINF, TL, TH)
IMPLICIT NONE
INTEGER,PARAMETER:: RK=MY_RKIND
INTEGER,    INTENT(IN):: NFILE
INTEGER,    INTENT(IN):: N ! DEGREE OF THE FILTER.
COMPLEX(RK),INTENT(IN):: ZPOLE(N), ZCOEF(N) ! POLES AND THEIR COEFFICIENTS.
REAL(RK),   INTENT(IN):: CINF  ! COEFFICIENT OF THE CONSTANT TERM.
REAL(RK),   INTENT(IN):: TL, TH ! INTERVAL OF ABSCISSAS FOR PLOT.
!-----------------------------------------
COMPLEX(RK),ALLOCATABLE:: ZP_RED(:), ZC_RED(:)
REAL(RK) CINF_RED
INTEGER DIGITS, DHIGH, DLOW, DSTEP
INTEGER J
!-----------------------------------------
  ALLOCATE (ZP_RED(N), ZC_RED(N))
  !--
  SELECT CASE(KIND(1.0_RK))
  CASE(KIND(1.E0))
    DHIGH=8;  DLOW=2; DSTEP=-2
  CASE(KIND(1.D0))
    DHIGH=15; DLOW=3; DSTEP=-3
#ifndef NO_QUADRUPLE_PRECISION
  CASE(KIND(1.Q0))
    DHIGH=30; DLOW=5; DSTEP=-5
#endif
  END SELECT
  !--
  DO  DIGITS = DHIGH, DLOW, DSTEP
    WRITE (NFILE, 100) DIGITS
    100 FORMAT(/'"D=',I3,'"')
    ZP_RED(:) = ZPOLE(:)
    ZC_RED(:) = ZCOEF(:)
    CINF_RED = CINF ! IS THIS OK?
    DO  J = 1, N
      CALL Z_REDUCE_ACCURACY (ZP_RED(J), DIGITS)
      CALL Z_REDUCE_ACCURACY (ZC_RED(J), DIGITS)
    ENDDO
    CALL PLOT_EXPANSION (NFILE, N, ZP_RED, ZC_RED, CINF_RED, TL, TH)
  ENDDO
  DEALLOCATE (ZP_RED, ZC_RED)
END SUBROUTINE PLOT_REDUCED_EXPANSION
