!******************************************************************************
!
! $Id: chebyshev.F90,v 1.1 2010/07/10 01:42:42 murakami Exp $
!
!******************************************************************************
!==============================================================================
! THE CHEBYSHEV POLYNOMIAL OF THE FIRST KIND IN COMPLEX ARGUMENT.
!==============================================================================
FUNCTION ZCHEB (N, Z) RESULT(VAL)
IMPLICIT NONE
INTEGER,PARAMETER:: RK = MY_RKIND
COMPLEX(RK) VAL  ! FUNCTION'S RETURN VALUE.
INTEGER,    INTENT(IN):: N  ! DEGREE OF POLYNOMIAL.
COMPLEX(RK),INTENT(IN):: Z  ! COMPLEX ARGUMENT.
!--------------------------------
COMPLEX(RK) Y0, Y1, Y2
INTEGER J
!--------------------------
  IF (N < 0) THEN
    STOP 'ZCHEB: ERROR, N < 0.'
  ELSEIF (N == 0) THEN
    VAL = 1.0_RK
  ELSEIF (N == 1) THEN
    VAL = Z
  ELSE
    Y0 = 1.0_RK
    Y1 = Z
    DO  J = 2, N
      Y2 = (2 * Z) * Y1 - Y0
      Y0 = Y1
      Y1 = Y2
    ENDDO
    VAL = Y2
  END IF
END FUNCTION ZCHEB
!==============================================================================
!  THE CHEBYSHEV POLYNOMIAL OF THE SECOND KIND IN THE COMPLEX ARGUMENT.
!==============================================================================
FUNCTION ZCHEB2ND (N, Z) RESULT(VAL)
IMPLICIT NONE
INTEGER,PARAMETER:: RK = MY_RKIND
COMPLEX(RK) VAL ! FUNCTION'S RETURN VALUE.
INTEGER,    INTENT(IN):: N ! THE DEGREE OF POLYNOMIAL.
COMPLEX(RK),INTENT(IN):: Z ! COMPLEX ARGUMENT.
!-----------------------------------
COMPLEX(RK) Y0, Y1, Y2
INTEGER J
!-------------------------
  IF (N < 0) THEN
    STOP 'ZCHEB2ND: ERROR, N < 0.'
  ELSEIF (N == 0) THEN
    VAL = 1.0_RK
  ELSE
    Y0 = 0.0_RK
    Y1 = 1.0_RK
    DO  J = 1, N
      Y2 = (2 * Z) * Y1 - Y0
      Y0 = Y1
      Y1 = Y2
    ENDDO
    VAL = Y2
  ENDIF
END FUNCTION ZCHEB2ND
!==============================================================================
!  THE CHEBYSHEV POLYNOMIAL OF THE FIRST KIND IN THE REAL ARGUMENT.
!==============================================================================
FUNCTION CHEB(N, X) RESULT(VAL)
IMPLICIT NONE
INTEGER,PARAMETER:: RK = MY_RKIND
REAL(RK) VAL ! FUNCTION'S RETURN VALUE.
INTEGER, INTENT(IN):: N ! DEGREE OF POLYNOMIAL.
REAL(RK),INTENT(IN):: X ! REAL ARGUMENT.
!--------------------------------
REAL(RK) Y0, Y1, Y2
INTEGER J
!--------------------
  IF (N < 0) THEN
    STOP 'CHEB: ERROR N < 0.'
  ELSEIF (N == 0) THEN
    VAL = 1.0_RK
  ELSEIF (N == 1) THEN
    VAL = X
  ELSE
    Y0 = 1.0_RK
    Y1 = X
    DO  J = 2, N
      Y2 = (2 * X) * Y1 - Y0
      Y0 = Y1
      Y1 = Y2
    ENDDO
    VAL = Y2
  ENDIF
END FUNCTION CHEB
