!******************************************************************************
!
! $Id: filter_butter.F90,v 1.1 2010/07/10 01:42:42 murakami Exp $
!
!******************************************************************************
!==============================================================================
! CALCULATES THE VALUE OF "A_STOP", THE TRUE MINIMUM ATTENUATION IN THE STOP 
! BAND OF THE BUTTERWORTH FILTER.
!==============================================================================
SUBROUTINE FILTER_ASTOP_BUTTER (EPS, MU, N, A_STOP)
IMPLICIT NONE
INTEGER,PARAMETER:: RK = MY_RKIND
REAL(RK),INTENT(IN):: EPS, MU  ! "EPS", "MU" IN THE FORMULA OF ATTENUATION.
INTEGER, INTENT(IN):: N        ! DEGREE OF THE FILTER.
REAL(RK),INTENT(OUT):: A_STOP  ! CALCULATED VALUE OF "A_STOP"
!-----------------------------------
  A_STOP = 1 + EPS**2 * MU**(2*N)
END SUBROUTINE FILTER_ASTOP_BUTTER
!==============================================================================
!   CALCULATES THE VALUES OF "EPS" AND "N_MIN" OF THE BUTTERWORTH FILTER
!   FROM THE VALUES OF "A_MAX", "A_MIN".
!==============================================================================
SUBROUTINE FILTER_SETUP_BUTTER (MU, A_MAX, A_MIN, EPS, N_MIN)
IMPLICIT NONE
INTEGER,PARAMETER:: RK = MY_RKIND
REAL(RK),INTENT(IN):: MU, A_MAX, A_MIN ! THREE PARAMETERS FOR THE ATTENUATION.
REAL(RK),INTENT(OUT):: EPS   ! "EPS" IN THE FILTER'S FORMULA.
INTEGER,INTENT(OUT) :: N_MIN ! MINIMUM DEGREE OF FILTER TO SATISFY THE PROPERTY.
!--------------------------------------
REAL(RK) L_MIN
!--------------------------
  EPS  = SQRT(A_MAX-1)
  L_MIN = SQRT((A_MIN-1)/(A_MAX-1))
  N_MIN = CEILING(LOG(L_MIN)/LOG(MU))
END SUBROUTINE FILTER_SETUP_BUTTER
!===============================================================================
!   CALCULATES THE POLES AND THEIR COEFFICIENTS OF THE BUTTERWORTH FILTER 
!   IN THE STANDARD COORDINATE.
!   NOTE, ONLY THOSE POLES WHOSE IMAGINARY PARTS ARE POSITIVES AND THEIR 
!   COEFFICIENTS ARE CALCULATED.
!===============================================================================
SUBROUTINE FILTER_POLES_COEFS_BUTTER (N, EPS, ZPOLE, ZCOEF, CINF)
IMPLICIT NONE
INTEGER,PARAMETER:: RK = MY_RKIND
INTEGER, INTENT(IN):: N   ! DEGREE OF THE FILTER "N".
REAL(RK),INTENT(IN):: EPS ! VALUE OF "EPS" IN THE FORMULA OF BUTTERWORTH FILTER.
COMPLEX(RK),INTENT(OUT):: ZPOLE(N) ! POLES WITH POSITIVE IMAGINARY PARTS.
COMPLEX(RK),INTENT(OUT):: ZCOEF(N) ! CORRESPONDING COEFFICIENTS OF THE POLES.
REAL(RK),   INTENT(OUT):: CINF     ! COEFFICIENTS OF CONSTANT TERM.
!------------------------------------------------------------------
REAL(RK),PARAMETER:: PI = 4*ATAN(1.0_RK)
REAL(RK) THETA
INTEGER J
!--------------------------------------
  DO  J = 1, N
    THETA = (2*J-1)*PI/(2*N)
    ZPOLE(J) = (1/EPS)**(1.0_RK/N)*CMPLX(COS(THETA),SIN(THETA),KIND=RK)
    ZCOEF(J) = (-1.0_RK)/(2*N)*ZPOLE(J)
  ENDDO
  CINF = 0.0_RK
END SUBROUTINE FILTER_POLES_COEFS_BUTTER
