!******************************************************************************
!
! $Id: filter_cheb.F90,v 1.1 2010/07/10 01:42:42 murakami Exp $
!
!******************************************************************************
!==============================================================================
! CALCULATES THE VALUE OF "A_STOP", THE TRUE MINIMUM ATTENUATION IN THE STOP 
! BAND OF THE CHEBYSHEV FILTER.
!==============================================================================
SUBROUTINE FILTER_ASTOP_CHEB (EPS, MU, N, A_STOP)
IMPLICIT NONE
INTEGER,PARAMETER:: RK = MY_RKIND
REAL(RK),INTENT(IN):: EPS, MU ! "EPS", "MU" IN THE FORMULA OF ATTENUATION.
INTEGER, INTENT(IN):: N ! DEGREE OF FILTER.
REAL(RK),INTENT(OUT):: A_STOP ! THE VALUE OF "A_STOP".
!----------------------------
REAL(RK),EXTERNAL:: CHEB
!------------------------
  A_STOP = 1 + EPS**2 * CHEB(N, MU)**2
END SUBROUTINE FILTER_ASTOP_CHEB
!==============================================================================
! FROM THE VALUES OF "A_MAX", "A_MIN" AND "MU", 
! CALCULATE THE VALUES "EPS" AND "N_MIN" OF THE CHEBYSHEV FILTER.
!==============================================================================
SUBROUTINE FILTER_SETUP_CHEB (MU, A_MAX, A_MIN, EPS, N_MIN)
IMPLICIT NONE
INTEGER,PARAMETER:: RK = MY_RKIND
REAL(RK),INTENT(IN):: MU, A_MAX, A_MIN ! THREE SHAPE PARAMETERS FOR ATTENUATION.
REAL(RK),INTENT(OUT):: EPS   ! "EPS" IN THE FORMULA OF ATTENUATION.
INTEGER,INTENT(OUT):: N_MIN  ! MINIMAL DEGREE OF THE FILTER.
!-----------------------------
! INTRINSIC ACOSH ! INVERSE HYPERBOLIC-COSINE (FORTRAN2008).
REAL(RK) L_MIN
!-----------------------------
  EPS = SQRT(A_MAX-1)
  L_MIN = SQRT((A_MIN-1) / (A_MAX-1))
  N_MIN = CEILING(ACOSH(L_MIN) / ACOSH(MU))
END SUBROUTINE FILTER_SETUP_CHEB
!==============================================================================
! CALCULATES THE POLES AND THEIR COEFFICIENTS OF THE CHEBYSHEV FILTER IN THE
! STANDARD COORDINATE.
! NOTE, ONLY THOSE POLES WHOSE IMAGINARY PARTS ARE POSITIVES AND THEIR 
! COEFFICIENTS ARE CALCULATED.
!==============================================================================
SUBROUTINE FILTER_POLES_COEFS_CHEB (N, EPS, ZPOLE, ZCOEF, CINF)
IMPLICIT NONE
INTEGER,PARAMETER:: RK = MY_RKIND
INTEGER, INTENT(IN):: N   ! DEGREE OF THE FILTER.
REAL(RK),INTENT(IN):: EPS ! VALUE OF "EPS" IN THE FORMULA OF ATTENUATION.
COMPLEX(RK),INTENT(OUT):: ZPOLE(N) ! POLES WITH POSITIVE IMAGINARY PARTS,
COMPLEX(RK),INTENT(OUT):: ZCOEF(N) ! AND THEIR COEFFICIENTS.
REAL(RK),   INTENT(OUT):: CINF     ! THE COEFFICIENT OF CONSTANT TERM.
!------------------------------------------------------------------------------
! INTRINSIC ASINH ! INVERSE HYPERBOLIC-SINE (FORTRAN2008).
REAL(RK),PARAMETER:: PI = 4*ATAN(1.0_RK)
COMPLEX(RK),EXTERNAL:: ZCHEB, ZCHEB2ND
REAL(RK) TAU, THETA
INTEGER J
!-------------------------------------------------------
  TAU = ASINH(1.0_RK/EPS) / N
  DO  J = 1, N
    THETA = (2*J-1)*PI/(2*N)
    ZPOLE(J) = CMPLX(COSH(TAU)*COS(THETA), SINH(TAU)*SIN(THETA), KIND=RK)
    ZCOEF(J) = (-0.5_RK)*ZCHEB(N, ZPOLE(J))/(N*ZCHEB2ND(N-1,ZPOLE(J)))
  ENDDO
  CINF = 0.0_RK ! THIS IS ALWAYS ZERO.
END SUBROUTINE FILTER_POLES_COEFS_CHEB
