!******************************************************************************
!
! $Id: main_std.F90,v 1.3 2010/07/12 12:36:15 murakami Exp $
!
!******************************************************************************
!==============================================================================
! FOR THE STANDARD EIGENPROBLEM:
!
!  A TEST PROGRAM OF THE FILTER DIAGONALIZATION METHOD FOR THE BANDED 
!  SYMMETRIC STANDARD EIGENPROBLEM "Ax=\lambda x".
!  THE FILTER IS A LINEAR COMBINATION OF RESOLVENTS.
!
!==============================================================================
PROGRAM MAIN_STD
IMPLICIT NONE
INTEGER,PARAMETER:: RK = MY_RKIND
INTEGER  NPROB        ! PROBLEM NUMBER.
INTEGER  N            ! SIZE OF MATRIX "A" OF THE EIGENPROBLEM.
INTEGER  NLD          ! LOWER BANDWIDTH OF MATRIX "A".
INTEGER  FILT_TYPE    ! TYPE OF FILTER.
REAL(RK) MU           ! SHAPE PARAMETER OF ATTENUATION OF THE FILTER.
REAL(RK) A_MAX, A_MIN ! UPPER AND LOWER BOUNDS OF ATTENUATION OF THE FILTER.
INTEGER  NF           ! (HALF) DEGREE OF THE FILTER.
REAL(RK) LAM0, LAM1   ! BOTH ENDS OF THE INTERVAL OF EIGENVALUE TO SOLVE.
INTEGER  NV           ! NUMBER OF INPUT VECTORS TO THE FILTER.
INTEGER  MX_ITER      ! MAX NUMBER OF INVERSE ITERATIONS FOR CORRECTION.
REAL(RK),ALLOCATABLE:: BAND_A(:,:) ! STORE OF BANDED SYMMETRIC MATRIX "A".
REAL(RK),ALLOCATABLE:: X(:,:) ! SET OF INPUT VECTORS TO THE FILTER.
REAL(RK),ALLOCATABLE:: Y(:,:) ! SET OF VECTORS.
COMPLEX(RK),ALLOCATABLE:: ZPOLE(:) ! NORMALIZED POLES OF THE FILTER.
COMPLEX(RK),ALLOCATABLE:: ZCOEF(:) ! NORMALIZED COEFFICIENTS OF THE FILTER.
REAL(RK)               :: CINF ! COEFFICIENT FOR THE CONSTANT TERM(NOT IN USE).
REAL(RK)   ,ALLOCATABLE:: SIGMA(:) ! SINGULAR VALUES BY SVD.
REAL(RK)   ,ALLOCATABLE:: R_DIAG(:)! DIAGONALS OF THE "R" MATRIX BY H-QR.
REAL(RK)   ,ALLOCATABLE:: Q(:)     ! APPROXIMATED EIGENVALUES.
REAL(RK)   ,ALLOCATABLE:: MAGIN(:), MAGOUT(:) ! ROUND-OFF MAGNIFICATION RATES.
INTEGER  IFLAG ! STATUS FLAG FOR MEMORY ALLOCATION.
!---
INTEGER,PARAMETER:: FILE_66 = 66
INTEGER,PARAMETER:: FILE_77 = 77
REAL(RK) REL_THRES  ! RELATIVE THRESHOLD FOR CUT-OFF.
INTEGER  RANK_Y     ! THE EFFECTIVE RANK OF VECTORS Y DETERMINED.
INTEGER  L
REAL*8   TM(0:6)    ! ARRAY OF THE WALL-CLOCK TIMES.
CHARACTER(80) FMT200, FMT300
CHARACTER(30) STR_PREC
INTEGER NUM_FOUND
REAL(RK) EPS_A
!------------------------------------------------------------------------------
  WRITE(*,      *) 'PROG. MAIN_STD (STANDARD EIGENPROBLEM).'
  WRITE(FILE_66,*) 'PROG. MAIN_STD (STANDARD EIGENPROBLEM).'
  !------
  CALL COMPARE_RK ! CHECK IF KIND-VALUE "RK" IS SAME TO THAT OF FILTER LIBRARY.
  !------
  CALL SET_STR_PREC ! SET THE STRING "STR_PREC"
  CALL SET_FMT200   ! SET THE FORMAT STRING "FMT200".
  WRITE(*,      100) RK, STR_PREC
  WRITE(FILE_66,100) RK, STR_PREC
  100 FORMAT('RK=',I2, 1X, A)
  !====================
  ! INPUT THE JOB PARAMETERS.
  CALL SET_JOB_PARAMS (NPROB, N, NLD, LAM0, LAM1, &
                       FILT_TYPE, MU, A_MAX, A_MIN, NF, &
                       NV, REL_THRES, &
                       MX_ITER)
  !====================
  ! ALLOCATE FILTER PARAMETERS.
  ALLOCATE (ZPOLE(NF), ZCOEF(NF), STAT=IFLAG)
    IF (IFLAG/=0) STOP 'PROG. MAIN_STD: FAILED TO ALLOCATE ZPOLE, ZCOEF.'
  !====================
  ! CALCULATE THE POLES AND THEIR COEFFICIENTS FOR THE FILTER.
  EPS_A = SQRT(A_MAX-1)
  CALL FILTER_POLES_COEFS (FILT_TYPE, NF, MU, EPS_A, ZPOLE, ZCOEF, CINF)
  !====================
  ! ALLOCATE THE BAND MATRIX.
  ALLOCATE (BAND_A(NLD+1,N), STAT=IFLAG)
    IF (IFLAG/=0) STOP 'PROG. MAIN_STD: FAILED TO ALLOCATE BAND_A.'
  ! ALLOCATE THE RANDOM VECTORS.
  ALLOCATE (X(N,NV), STAT=IFLAG)
    IF (IFLAG/=0) STOP 'PROG. MAIN_STD: FAILED TO ALLOCATE X.'
  ! ALLOCATE THE FILTERED VECTORS.
  ALLOCATE (Y(N,NV), STAT=IFLAG)
    IF (IFLAG/=0) STOP 'PROG. MAIN_STD: FAILED TO ALLOCATE Y.'
  ! ALLOCATE THE VECTORS OF THE ERROR MAGNIFICATION RATES.
  ALLOCATE (MAGIN(NV), MAGOUT(NV), STAT=IFLAG)
    IF (IFLAG/=0) STOP 'PROG. MAIN_STD: FAILED TO ALLOCATE MAGIN, MAGOUT.'
  !====================
  ! SET THE VALUES OF THE BAND MATRIX A.
  CALL SET_BAND_A (NPROB, N, NLD, BAND_A)
  !------
  CALL MY_CLOCK (TM(0))
  !====================
  ! GENERATE X, THE RANDOM INITIAL VECTORS.
  DO  L = 1, NV
    CALL RANDOM_VECTOR (N, X(1,L))
  ENDDO
  !------
  CALL MY_CLOCK (TM(1))
  !====================
  ! ORTHONORMALIZE "X" (NEED NOT BE SO ACCURATE).
  CALL ORTH_MGS (N, NV, X)
  PRINT *, 'X IS ORTHONORMALIZED.'
  !------
  CALL MY_CLOCK (TM(2))
  !====================
  ! Y <-- FILTER (X)
  CALL APPLY_FILTER_NV (NF, ZPOLE, ZCOEF, CINF, &
                        LAM0, LAM1, &
                        BAND_A, N, NLD, SIZE(BAND_A,1), &
                        X, Y, NV)
  DEALLOCATE (X)
  CALL MY_CLOCK (TM(3))
  !====================
  ! ORTHONORMALIZE THE SET OF "NV" VECTORS "Y", THE OUTPUT FROM THE FILTER.
#define USE_HQR
#undef  USE_HQR
#ifdef USE_HQR
  ! BY THE HOUSEHOLDER-"QR" METHOD WITH COLUMN PIVOTING.
  ALLOCATE (R_DIAG(NV), STAT=IFLAG)
    IF (IFLAG/=0) STOP 'PROG. MAIN_STD: FAILED TO ALLOCATE R_DIAG.'
  CALL ORTH_HQR (N, NV, Y, REL_THRES, R_DIAG, RANK_Y)
  WRITE(*,       *) 'ORTHONORMALIZED BY THE HOUSEHOLDER QR.'
  WRITE(FILE_66, *) 'ORTHONORMALIZED BY THE HOUSEHOLDER QR.'
  MAGIN(:RANK_Y) = 1 / R_DIAG(:RANK_Y)
#else
  ! BY THE CLASSICAL SINGULAR VALUE DECOMPOSITION (WITH CORRECTION).
  ALLOCATE (SIGMA(NV), STAT=IFLAG)
    IF (IFLAG/=0) STOP 'PROG. MAIN_STD: FAILED TO ALLOCATE SIGMA.'
  CALL ORTH_SVD (N, NV, Y, REL_THRES, SIGMA, RANK_Y)
  WRITE(*,       *) 'ORTHONORMALIZED BY THE CLASSICAL SVD.'
  WRITE(FILE_66, *) 'ORTHONORMALIZED BT THE CLASSICAL SVD.'
  MAGIN(:RANK_Y) = 1 / SIGMA(:RANK_Y)
#endif
  CALL MY_CLOCK (TM(4))
  !====================
  ! THE SUBSPACE METHOD TO OBTAIN THE APPROXIMATED EIGENPAIRS.
  ALLOCATE (Q(RANK_Y), STAT=IFLAG)
    IF (IFLAG/=0) STOP 'PROG. MAIN_STD: FAILED TO ALLOCATE Q.'
  !------
  CALL RITZ (BAND_A, N, NLD, SIZE(BAND_A,1), &
             RANK_Y, Y, Q, LAM0, LAM1, MAGIN, MAGOUT)
  !------
  CALL MY_CLOCK (TM(5))
  !====================
  ! INVERSE ITERATION PROCESS TO IMPROVE THE "RANK_Y" APPROXIMATED EIGENPAIRS.
  CALL INV_ITER_STEP (BAND_A, N, NLD, SIZE(BAND_A,1), &
                      RANK_Y, Y, Q, MX_ITER)
  !------
  CALL MY_CLOCK (TM(6))
  !====================
  ! REPORT THE APPROXIMATED EIGENVALUES, AND IF THEY ARE IN THE INTERVAL.
  WRITE(      *, *) 'AFTER THE INVERSE ITERATIONS.'
  WRITE(FILE_66, *) 'AFTER THE INVERSE ITERATIONS.'
  WRITE(      *, *) '    THE VALUE OF Q;  IS IT IN THE INTERVAL?'
  WRITE(FILE_66, *) '    THE VALUE OF Q;  IS IT IN THE INTERVAL?'
  DO  L = 1, RANK_Y
    WRITE(      *, FMT200) L, Q(L), (LAM0<=Q(L).AND.Q(L)<=LAM1)
    WRITE(FILE_66, FMT200) L, Q(L), (LAM0<=Q(L).AND.Q(L)<=LAM1)
  ENDDO
  !------
  NUM_FOUND = COUNT(LAM0<=Q(:).AND.Q(:)<=LAM1)
  SELECT CASE(NUM_FOUND)
    CASE(0:1)   ; FMT300 ="('## FOUND ', I5, ' SOLUTION.')"
    CASE DEFAULT; FMT300 ="('## FOUND ', I5, ' SOLUTIONS.')"
  END SELECT
  WRITE(      *, FMT300) NUM_FOUND
  WRITE(FILE_66, FMT300) NUM_FOUND
  !------
  WRITE(      *, *) 'PROG. MAIN_STD: NORMAL END.'
  WRITE(FILE_66, *) 'PROG. MAIN_STD: NORMAL END.'
  !====================
  ! REPORT THE ELAPSED TIMES.
  CALL PRINTF_TIME (FILE_77, "THE WHOLE FDM PROCESS    ", TM(5)-TM(0))
  !------
  CALL PRINTF_TIME (FILE_77, "  GENERATE RANDOM VECTORS", TM(1)-TM(0))
  CALL PRINTF_TIME (FILE_77, "  ORTHONORMALIZE VECTORS ", TM(2)-TM(1))
  CALL PRINTF_TIME (FILE_77, "  FILTER INPUT VECTORS   ", TM(3)-TM(2))
#ifdef USE_HQR
  CALL PRINTF_TIME (FILE_77, "  HQR OUTPUT VEC ANALYSIS", TM(4)-TM(3))
#else
  CALL PRINTF_TIME (FILE_77, "  SVD OUTPUT VEC ANALYSIS", TM(4)-TM(3))
#endif
  CALL PRINTF_TIME (FILE_77, "  CALCULATE RITZ VALUES  ", TM(5)-TM(4))
  CALL PRINTF_TIME (FILE_77, "INVERSE-ITERATION PROCESS", TM(6)-TM(5))
  !------
  STOP
CONTAINS
  !--------------------------
  SUBROUTINE SET_STR_PREC
    SELECT CASE(RK)
      CASE(KIND(1.E0)); STR_PREC = '(SINGLE PRECISION)'
      CASE(KIND(1.D0)); STR_PREC = '(DOUBLE PRECISION)'
#ifndef NO_QUADRUPLE_PRECISION
      CASE(KIND(1.Q0)); STR_PREC = '(QUADRUPLE PRECISION)'
#endif
    END SELECT
  END SUBROUTINE
  !--------------------------
  SUBROUTINE SET_FMT200
    SELECT CASE(RK)
      CASE(KIND(1.E0)); FMT200 = "('Q(',I4,')=',ES14.7, ' : ',L)"
      CASE(KIND(1.D0)); FMT200 = "('Q(',I4,')=',ES22.15,' : ',L)"
#ifndef NO_QUADRUPLE_PRECISION
      CASE(KIND(1.Q0)); FMT200 = "('Q(',I4,')=',ES41.34,' : ',L)"
#endif
      CASE DEFAULT    ; FMT200 = "('Q(',I4,')=',ES22.15,' : ',L)"
    END SELECT
  END SUBROUTINE
  !--------------------------
END PROGRAM MAIN_STD
!==============================================================================
!
!    APPLY THE FILTER SIMULTANEOUSLY TO "NV" VECTORS.
!
!==============================================================================
SUBROUTINE APPLY_FILTER_NV (NF, ZPOLE, ZCOEF, CINF, &
                            LAM0, LAM1, &
                            BAND_A, N, NLD, NA, X, Y, NV)
IMPLICIT NONE
INTEGER,PARAMETER:: RK = MY_RKIND
INTEGER,    INTENT(IN):: NF ! NUMBER OF RESOLVENTS FOR THE FILTER.
COMPLEX(RK),INTENT(IN):: ZPOLE(NF) ! NORMALIZED POLES OF THE FILTER.
COMPLEX(RK),INTENT(IN):: ZCOEF(NF) ! NORMALIZED COEFFICIENTS OF THE FILTER.
REAL(RK),   INTENT(IN):: CINF      ! COEFFICIENT FOR IDENTITY(CURRENTLY UNUSED).
REAL(RK),INTENT(IN):: LAM0, LAM1   ! INTERVAL OF THE PASSBAND OF THE FILTER.
REAL(RK),INTENT(IN):: BAND_A(NA,N) ! STORE OF BANDED SYMMETRIC MATRIX "A".
INTEGER, INTENT(IN):: N            ! SIZE OF MATRIX "A".
INTEGER, INTENT(IN):: NLD          ! LOWER BANDWIDTH OF MATRIX "A".
INTEGER, INTENT(IN):: NA           ! LEADING DIMENSION OF "BAND_A".
REAL(RK),INTENT(IN) :: X(N,NV) ! SET OF INPUT VECTORS TO THE FILTER.
REAL(RK),INTENT(OUT):: Y(N,NV) ! SET OF OUTPUT VECTORS FROM THE FILTER.
INTEGER, INTENT(IN) :: NV      ! NUMBER OF VECTORS TO APPLY FOR THE FILTER.
!------------------------------------------------------------------------------
REAL(RK),PARAMETER:: EPS_LU = EPSILON(1.0_RK) * 10
COMPLEX(RK),ALLOCATABLE:: Z(:,:)      ! STORE FOR "NV" COMPLEX VECTORS.
COMPLEX(RK),ALLOCATABLE:: BAND_C(:,:) ! BANDED MATRIX "C" AND ITS "LU"-FACTORS.
INTEGER    ,ALLOCATABLE:: IPIV(:)     ! PIVOT PERMUTATION DATA.
INTEGER IFLAG ! STATUS FLAG FOR ALLOCATION.
COMPLEX(RK) ZRHO(NF), ZGAMMA(NF) ! SHIFTS AND COEFFICIENTS FOR RESOLVENTS.
REAL(RK) CNORM
INTEGER I
!----------------------------------------------------------
  ! MAKE THE SHIFTS OF RESOLVENTS AND THEIR COEFFICIENTS OF LINEAR COMBINATION.
  ! "CINF" IS NOT TRANSFORMED AS "ZGAMMA" IS (BUT IS LEFT UNUSED AND OMITTED).
  ZRHO(:) = ZPOLE(:) * ((LAM1-LAM0)/2) + ((LAM0+LAM1)/2)
  ZGAMMA(:) = ZCOEF(:) * ((LAM1-LAM0)/2)
  !====================
  ALLOCATE (Z(N,NV), STAT=IFLAG)
    IF (IFLAG/=0) STOP 'SUBR. APPLY_FILTER_NV: FAILED TO ALLOCATE Z.'
  ALLOCATE (BAND_C(3*NLD+1,N), IPIV(N), STAT=IFLAG)
    IF (IFLAG/=0) STOP 'SUBR. APPLY_FILTER_NV: FAILED TO ALLOCATE BAND_C, IPIV.'
  !====================
  Y(:N,:NV) = 0.0_RK ! ACCUMULATION IS MADE TO THIS VECTOR.
  ! THE COMPLEX CONJUGATE SYMMETRY IS USED.
  DO  I = 1, NF 
#if 1
    WRITE(*,100) I
    100 FORMAT('DEBUG: (APPLY_FILTER_NV) I=', I5)
#endif
    !====================
    ! DECOMPOSE THE BAND MATRIX "(A - ZRHO(I) * IDENT)" TO "LU"-FACTORS.
    ! FIRST, MAKE THE SHIFTED BAND MATRIX "C := (A - ZRHO(I) * IDENT)".
    CALL BAND_SHIFT_C_STD (BAND_A, N, NLD, SIZE(BAND_A,1), &
                           ZRHO(I), BAND_C, SIZE(BAND_C,1), CNORM)
    ! THEN, DECOMPOSE THE BANDED MATRIX "C" TO THE "LU"-FACTORS.
    CALL BAND_DECOMP_C (N, NLD, EPS_LU, CNORM, BAND_C, SIZE(BAND_C,1), IPIV)
    !====================
    ! SIMULTANEOUSLY SOLVE THE LINEAR EQUATION: "C * Z = X"
    ! BY USING THE PREVIOUS "LU"-FACTORS OF "C".
    Z(:N,:NV) = X(:N,:NV)
    CALL BAND_LU_SOLVE_C_NV (N, NLD, NV, Z, BAND_C, SIZE(BAND_C,1), IPIV)
    !====================
    ! ADD THE REAL PARTS OF THE PRODUCT OF "ZGAMMA(I)" AND VECTOR "Z".
    ! THE FACTOR TWO COMES FROM THE SUM OF COMPLEX CONJUGATE PAIR.
    Y(:N,:NV) = Y(:N,:NV) + &
        (REAL((2*ZGAMMA(I)),KIND=RK) * REAL(Z(:N,:NV),KIND=RK) &
        -AIMAG((2*ZGAMMA(I)))        * AIMAG(Z(:N,:NV)))
    !====================
  ENDDO
  !------
  DEALLOCATE (Z)
  DEALLOCATE (BAND_C, IPIV)
END SUBROUTINE APPLY_FILTER_NV
!==============================================================================
!
!    INVERSE ITERATION PROCESS FOR "NV" APPROXIMATED EIGENPAIRS OF THE
!    STANDARD EIGENPROBLEM.
!
!==============================================================================
SUBROUTINE INV_ITER_STEP (BAND_A, N, NLD, NA, NV, Y, E, MX_ITER)
IMPLICIT NONE
INTEGER,PARAMETER:: RK = MY_RKIND
REAL(RK),INTENT(IN):: BAND_A(NA,N) ! STORE OF BANDED SYMMETRIC MATRIX "A".
INTEGER, INTENT(IN):: N            ! SIZE OF MATRIX "A".
INTEGER, INTENT(IN):: NLD          ! LOWER BANDWIDTH OF "A".
INTEGER, INTENT(IN):: NA           ! LEADING DIMENSION OF "BAND_A".
INTEGER, INTENT(IN):: NV         ! NUMBER OF APPROXIMATED EIGENPAIRS.
REAL(RK),INTENT(INOUT):: Y(N,NV) ! SET OF APPROXIMATED EIGENVECTORS.
REAL(RK),INTENT(INOUT):: E(NV)   ! SET OF APPROXIMATED EIGENVALUES.
INTEGER, INTENT(IN):: MX_ITER ! MAX NUMBER OF INVERSE ITERATIONS.
!------------------------------------------------------------------------------
INTEGER,PARAMETER:: FILE_99 = 99
REAL(RK),ALLOCATABLE:: BAND_C(:,:) ! STORE FOR THE "LU"-FACTORS.
INTEGER ,ALLOCATABLE:: IPIV(:)     ! STORE FOR THE PIVOT PERMUTATION DATA.
INTEGER IFLAG ! STATUS FLAG FOR ALLOCATION.
INTEGER L
!----------------------------------------------------------
  ALLOCATE (BAND_C(3*NLD+1,N), STAT=IFLAG)
    IF (IFLAG/=0) STOP 'SUBR. INV_ITER_STEP: FAILED TO ALLOCATE BAND_C.'
  ALLOCATE (IPIV(N), STAT=IFLAG)
    IF (IFLAG/=0) STOP 'SUBR. INV_ITER_STEP: FAILED TO ALLOCATE IPIV.'
  WRITE(      *, *) 'INVERSE ITERATION PROCESS:'
  WRITE(FILE_99, *) 'INVERSE ITERATION PROCESS:'
  !====================
  DO  L = 1, NV
    WRITE(      *, 100) L
    WRITE(FILE_99, 100) L
    100 FORMAT('L=', I4)
    CALL INV_ITR (BAND_A, N, NLD, SIZE(BAND_A,1), &
                  E(L), Y(:,L), BAND_C, SIZE(BAND_C,1), IPIV, MX_ITER)
  ENDDO
  !====================
  !------
  DEALLOCATE (BAND_C, IPIV)
END SUBROUTINE INV_ITER_STEP
!==============================================================================
!
!  INVERSE ITERATION PROCESS FOR AN APPROXIMATED EIGENPAIR OF THE STANDARD
!  EIGENPROBLEM.
!
!==============================================================================
SUBROUTINE INV_ITR (BAND_A, N, NLD, NA, &
                    Q, V, BAND_C, NC, IPIV, &
                    MX_ITER)
IMPLICIT NONE
INTEGER,PARAMETER:: RK = MY_RKIND
REAL(RK),INTENT(IN):: BAND_A(NA,N) ! STORE OF BANDED SYMMETRIC MATRIX "A".
INTEGER, INTENT(IN):: NA           ! LEADING DIMENSION OF "BAND_A".
INTEGER, INTENT(IN):: N            ! SIZE OF MATRIX "A".
INTEGER, INTENT(IN):: NLD          ! LOWER BANDWIDTH OF "A".
REAL(RK),INTENT(INOUT):: Q    ! AN APPROXIMATED EIGENVALUE.
REAL(RK),INTENT(INOUT):: V(N) ! AN APPROXIMATED EIGENVECTOR.
REAL(RK)           :: BAND_C(NC,N) ! WORK AREA FOR "LU"-FACTORS.
INTEGER, INTENT(IN):: NC           ! LEADING DIMENSION OF "BAND_C".
INTEGER            :: IPIV(N)      ! WORK AREA FOR PIVOT PERMUTATION DATA.
INTEGER, INTENT(IN):: MX_ITER  ! MAX NUMBER OF INVERSE ITERATIONS.
!------------------------------------------------------------------------------
REAL(RK),PARAMETER:: EPS_LU = EPSILON(1.0_RK) * 10
INTEGER,PARAMETER:: FILE_99 = 99
REAL(RK),ALLOCATABLE:: R(:)
INTEGER IFLAG ! STATUS FLAG FOR ALLOCATION.
REAL(RK) MAG, DEL, CNORM
INTEGER ITER
CHARACTER(40) FMT
!----------------------------------------------------------
  ALLOCATE (R(N), STAT=IFLAG)
  IF (IFLAG/=0) STOP 'INV_ITR: FAILED TO ALLOCATE R AND S.'
  SELECT CASE (RK)
    CASE(KIND(1.E0)); FMT = "(1X, 'Q=', ES14.7)"
    CASE(KIND(1.D0)); FMT = "(1X, 'Q=', ES22.15)"
#ifndef NO_QUADRUPLE_PRECISION
    CASE(KIND(1.Q0)); FMT = "(1X, 'Q=', ES41.34)"
#endif
    CASE DEFAULT    ; FMT = "(1X, 'Q=', ES22.15)"
  END SELECT
  100 FORMAT(1X, 'IT', I1, ':')
  200 FORMAT(1X, 'DEL=', ES9.1 : 1X, 'MAG=', ES9.1)
  !====================
  LOOP: DO  ITER = 0, MX_ITER
    IF (ITER > 0) THEN
    ! SOLVE THE LINEAR EQUATION: "(A - Q * IDENT) V_NEXT = V".
    CALL BAND_SHIFT_STD (BAND_A, N, NLD, SIZE(BAND_A,1), &
                         Q, BAND_C, SIZE(BAND_C,1), CNORM)
    CALL BAND_DECOMP (N, NLD, EPS_LU, CNORM, BAND_C, SIZE(BAND_C,1), IPIV)
    CALL BAND_LU_SOLVE (N, NLD, V, BAND_C, NC, IPIV)
    ENDIF
    !====================
    ! NORMALIZE THE VECTOR "V".
    MAG = SQRT(SUM(V(:N)**2))
    V(:N) = V(:N) / MAG 
    !====================
    ! COMPUTE "R := A * V".
    CALL BAND_MULT (BAND_A, N, NLD, SIZE(BAND_A,1), V, R) 
    ! COMPUTE THE UPDATED RAYLEIGH QUOTIENT: "Q = (V, A*V)".
    Q = DOT_PRODUCT(V(:N), R(:N)) 
    !====================
    ! COMPUTE THE RESIDUAL VECTOR: "R := (A - Q * IDENT) V".
    R(:N) = R(:N) - Q * V(:N) 
    ! "DEL" IS THE 2-NORM OF THE RESIDUAL, AN ERROR BOUND OF THE EIGENVALUE.
    DEL = SQRT(SUM(R(:N)**2))
    !====================
    ! SHOW THE APPROX EIGENVALUE "Q", ERROR BOUND "DEL", AND THE MAGNIFICATION
    ! FACTOR "MAG" FOR THE EIGENVECTOR BY THE INVERSE ITERATION.
    WRITE(      *, 100, ADVANCE='NO') ITER
    WRITE(FILE_99, 100, ADVANCE='NO') ITER
    WRITE(      *, FMT, ADVANCE='NO') Q
    WRITE(FILE_99, FMT, ADVANCE='NO') Q
    IF (ITER == 0) THEN
      WRITE(      *, 200) DEL
      WRITE(FILE_99, 200) DEL
    ELSE
      WRITE(      *, 200) DEL, MAG
      WRITE(FILE_99, 200) DEL, MAG
    ENDIF
    !====================
  ENDDO LOOP
  !------
  DEALLOCATE (R)
END SUBROUTINE INV_ITR
