!******************************************************************************
!
! $Id: set_band_a.F90,v 1.2 2010/07/12 12:36:15 murakami Exp $
!
!******************************************************************************
!==============================================================================
!
!   SET THE VALUES OF THE BANDED MATRIX "A".
!
!==============================================================================
SUBROUTINE SET_BAND_A (NPROB, N, NLD, BAND_A)
IMPLICIT NONE
INTEGER,PARAMETER:: RK = MY_RKIND
INTEGER,INTENT(IN):: NPROB ! PROBLEM NUMBER.
INTEGER,INTENT(IN) :: N                ! SIZE OF MATRIX "A".
INTEGER,INTENT(IN) :: NLD              ! LOWER BANDWIDTH OF "A".
REAL(RK),INTENT(OUT):: BAND_A(NLD+1,N) ! STORE OF BANDED SYMMETRIC MATRIX "A",
                                       ! WHERE A(I,J)=BAND_A(M+J-I,I).
!-----------------------------------------------------------------------
  SELECT CASE(NPROB)
    CASE(1); CALL SET_BAND_A_1 (N, NLD, BAND_A)
    CASE(2); CALL SET_BAND_A_2 (N, NLD, BAND_A)
    CASE(3); CALL SET_BAND_A_3 (N, NLD, BAND_A)
    CASE(4); CALL SET_BAND_A_4 (N, NLD, BAND_A)
    CASE DEFAULT; STOP 'SUBR. SET_BAND_A: UNKNOWN NPROB.'
  END SELECT
END SUBROUTINE SET_BAND_A
!==============================================================================
!
!   CASE NPROB=1 : SET THE VALUES OF THE BANDED MATRIX "A".
!   THE MATRIX ELEMENTS OF "A" INSIDE THE BANDWIDTH ARE GIVEN BY 
!   THE EXPRESSION: "A_{P,Q}=\max(P,Q)-1".
!
!==============================================================================
SUBROUTINE SET_BAND_A_1 (N, NLD, BAND_A)
IMPLICIT NONE
INTEGER,PARAMETER:: RK = MY_RKIND
INTEGER, INTENT(IN) :: N               ! SIZE OF MATRIX.
INTEGER, INTENT(IN) :: NLD             ! LOWER HALF BANDWIDTH.
REAL(RK),INTENT(OUT):: BAND_A(NLD+1,N) ! STORE OF BANDED SYMMETRIC MATRIX "A",
                                       ! WHERE A(I,J)=BAND_A(M+J-I,I).
!-----------------------------------------------------------------------
INTEGER M, I, J
#define A(I,J)BAND_A(M+J-I,I)
!-----------------------------
  M = NLD + 1
  DO  J = 1, N
    DO  I = J, MIN(J+NLD,N)
      A(I,J) = MAX(I,J) - 1.0_RK
    ENDDO
  ENDDO
#undef A
END SUBROUTINE SET_BAND_A_1
!==============================================================================
!
!   CASE NPROB=2 : SET THE VALUES OF THE BANDED MATRIX "A".
!   THE EIGENPAIRS OF ARE VERY CLOSE TO DUPLICATE IN THEIR EIGENVALUES.
!
!==============================================================================
SUBROUTINE SET_BAND_A_2 (N, NLD, BAND_A)
IMPLICIT NONE
INTEGER,PARAMETER:: RK = MY_RKIND
INTEGER,INTENT(IN)  :: N               ! SIZE OF MATRIX "A".
INTEGER,INTENT(IN)  :: NLD             ! LOWER BANDWIDTH OF "A".
REAL(RK),INTENT(OUT):: BAND_A(NLD+1,N) ! STORE OF BANDED SYMMETRIC MATRIX "A",
                                       ! WHERE A(I,J)=BAND_A(M+J-I,I).
!-----------------------------------------------------------------------
INTEGER,PARAMETER:: FILE_66 = 66
REAL(RK),PARAMETER:: SMALL_A = 1.E-4_RK
INTEGER CNT
INTEGER M, I, J
#define A(I,J)BAND_A(M+J-I,I)
!-----------------------------------------------------------------------
  WRITE(*,       100) SMALL_A
  WRITE(FILE_66, 100) SMALL_A
  100 FORMAT('SET_BAND_A_2: SMALL_A=', ES12.5)
  M = NLD + 1
  CNT = 0
  DO  J = 1, N/2
    DO  I = J, MIN(J+NLD,N/2)
      A(I,J) = MAX(I,J)-1.0_RK
      CNT = CNT + 1
    ENDDO
  ENDDO
  !---------------------------
  DO  J = MAX(1,(N/2+1)-NLD), (N/2)
    DO  I = (N/2+1), MIN(J+NLD,N)
      A(I,J) = SMALL_A
      CNT = CNT + 1
    ENDDO
  ENDDO
  !---------------------------
  DO  J = (N/2+1), N
    DO  I = J, MIN(J+NLD,N)
      A(I,J) = MAX(I-N/2,J-N/2)-1.0_RK
      CNT = CNT + 1
    ENDDO
  ENDDO
  WRITE(*,      200) CNT, (1+NLD)*(2*N-NLD)/2
  WRITE(FILE_66,200) CNT, (1+NLD)*(2*N-NLD)/2
  200 FORMAT('CNT=', I10, ',  (1+NLD)*(2*N-NLD)/2=', I10)
#undef A
END SUBROUTINE SET_BAND_A_2
!==============================================================================
!
!   CASE NPROB=3 : SET THE VALUES OF THE BANDED SYMMETRIC MATRIX "A".
!   THE MATRIX ELEMENTS OF "A" INSIDE THE BANDWIDTH ARE GIVEN BY 
!   THE EXPRESSION: "A_{P,Q}=P*Q/SQRT{P^2+Q^2}".
!
!==============================================================================
SUBROUTINE SET_BAND_A_3 (N, NLD, BAND_A)
IMPLICIT NONE
INTEGER,PARAMETER:: RK = MY_RKIND
INTEGER,INTENT(IN)  :: N               ! SIZE OF MATRIX "A".
INTEGER,INTENT(IN)  :: NLD             ! LOWER BANDWIDTH OF "A".
REAL(RK),INTENT(OUT):: BAND_A(NLD+1,N) ! STORE OF BANDED SYMMETRIC MATRIX "A",
                                       ! WHERE A(I,J)=BAND_A(M+J-I,I).
!-----------------------------------------------------------------------
INTEGER M, I, J
#define A(I,J)BAND_A(M+J-I,I)
!------------------------------
  M = NLD + 1
  DO  J = 1, N
    DO  I = J, MIN(J+NLD,N)
      A(I,J) = 1.0_RK*I*J/SQRT(REAL(I,KIND=RK)**2+REAL(J,KIND=RK)**2)
    ENDDO
  ENDDO
#undef A
END SUBROUTINE SET_BAND_A_3
!==============================================================================
!
!   CASE NPROB=4 : SET THE VALUES OF THE BANDED SYMMETRIC MATRIX "A".
!   THE MATRIX ELEMENTS OF "A" INSIDE THE BANDWIDTH ARE GIVEN BY 
!   THE EXPRESSION: "A_{P,Q}=SIN(2(P+Q))/SQRT{P^2+Q^2}".
!
!==============================================================================
SUBROUTINE SET_BAND_A_4 (N, NLD, BAND_A)
IMPLICIT NONE
INTEGER,PARAMETER:: RK = MY_RKIND
INTEGER,INTENT(IN)  :: N               ! SIZE OF MATRIX "A".
INTEGER,INTENT(IN)  :: NLD             ! LOWER BANDWIDTH OF "A".
REAL(RK),INTENT(OUT):: BAND_A(NLD+1,N) ! STORE OF BANDED SYMMETRIC MATRIX "A",
                                       ! WHERE A(I,J)=BAND_A(M+J-I,I).
!-----------------------------------------------------------------------
INTEGER M, I, J
#define A(I,J)BAND_A(M+J-I,I)
!------------------------------
  M = NLD + 1
  DO  J = 1, N
    DO  I = J, MIN(J+NLD,N)
      A(I,J) = COS(2.0_RK*(I+J))/SQRT(REAL(I,KIND=RK)**2+REAL(J,KIND=RK)**2)
    ENDDO
  ENDDO
#undef A
END SUBROUTINE SET_BAND_A_4
