!******************************************************************************
!
! $Id: set_band_ab.F90,v 1.1 2010/07/10 01:44:28 murakami Exp $
!
!******************************************************************************
!==============================================================================
!
!   SET THE VALUES OF THE BANDED MATRICES "A" AND "B".
!
!==============================================================================
SUBROUTINE SET_BAND_AB (NPROB, N, NLD, BAND_A, BAND_B)
IMPLICIT NONE
INTEGER,PARAMETER:: RK = MY_RKIND
INTEGER,INTENT(IN):: NPROB  ! PROBLEM NUMBER.
INTEGER,INTENT(IN)  :: N               ! SIZE OF BOTH MATRICES "A" AND "B".
INTEGER,INTENT(IN)  :: NLD             ! LOWER BANDWIDTH OF BOTH "A" AND "B".
REAL(RK),INTENT(OUT):: BAND_A(NLD+1,N) ! STORE OF BANDED SYMMETRIC MATRIX "A".
REAL(RK),INTENT(OUT):: BAND_B(NLD+1,N) ! STORE OF BANDED SYMMETRIC MATRIX "B".
                   ! WHERE "A(I,J)=BAND_A(M+J-I,I)", "B(I,J)=BAND_B(M+J-I,I)".
!-----------------------------------------------------------------------
  SELECT CASE(NPROB)
    CASE(1); CALL SET_BAND_AB_1 (N, NLD, BAND_A, BAND_B)
    CASE(2); CALL SET_BAND_AB_2 (N, NLD, BAND_A, BAND_B)
    CASE(3); CALL SET_BAND_AB_3 (N, NLD, BAND_A, BAND_B)
    CASE(4); CALL SET_BAND_AB_4 (N, NLD, BAND_A, BAND_B)
    CASE DEFAULT; STOP 'SUBR. SET_BAND_AB: ERROR UNKNOWN NPROB.'
  END SELECT
END SUBROUTINE SET_BAND_AB
!==============================================================================
!
!   NPROB=1 : SET THE VALUES OF THE BANDED MATRICES A AND B.
!   THE ELEMENTS OF MATRICES INSIDE THE BAND ARE GIVE BY THE EXPRESSIOS:
!   "A_{P,Q}=MAX{P,Q}-1", "B_{P,Q}=1/(P+Q-1)+DELTA_{P,Q}", 
!   WHERE "DELTA_{P,Q}" IS THE KRONECKER SYMBOL.
!
!==============================================================================
SUBROUTINE SET_BAND_AB_1 (N, NLD, BAND_A, BAND_B)
IMPLICIT NONE
INTEGER,PARAMETER:: RK = MY_RKIND
INTEGER, INTENT(IN) :: N               ! SIZE OF BOTH MATRICES "A" AND "B".
INTEGER, INTENT(IN) :: NLD             ! LOWER BANDWIDTH OF BOTH "A" AND "B".
REAL(RK),INTENT(OUT):: BAND_A(NLD+1,N) ! STORE OF BANDED SYMMETRIC MATRIX "A".
REAL(RK),INTENT(OUT):: BAND_B(NLD+1,N) ! STORE OF BANDED SYMMETRIC MATRIX "B".
                          ! "A(I,J)=BAND_A(M+J-I,I)", "B(I,J)=BAND_B(M+J-I,I)".
!-----------------------------------------------------------------------
INTEGER M, I, J
#define A(I,J)BAND_A(M+J-I,I)
#define B(I,J)BAND_B(M+J-I,I)
!-----------------------------
  M = NLD + 1
  DO  J = 1, N
    DO  I = J, MIN(J+NLD,N)
      A(I,J) = MAX(I,J)-1.0_RK
      B(I,J) = 1/(I+J-1.0_RK)
    ENDDO
    I = J
    B(I,J) = B(I,J)+1.0_RK
  ENDDO
#undef A
#undef B
END SUBROUTINE SET_BAND_AB_1
!==============================================================================
!
!   NPROB=2 : SET THE VALUES OF THE BANDED MATRICES "A" AND "B".
!   THE EIGENPAIRS ARE VERY NEAR TO DUPLICATE IN THEIR EIGENVALUES.
!
!==============================================================================
SUBROUTINE SET_BAND_AB_2 (N, NLD, BAND_A, BAND_B)
IMPLICIT NONE
INTEGER,PARAMETER:: RK = MY_RKIND
INTEGER, INTENT(IN) :: N               ! SIZE OF BOTH MATRICES "A" AND "B".
INTEGER, INTENT(IN) :: NLD             ! LOWER BANDWIDTH OF BOTH "A" AND "B".
REAL(RK),INTENT(OUT):: BAND_A(NLD+1,N) ! STORE OF BANDED SYMMETRIC MATRIX "A".
REAL(RK),INTENT(OUT):: BAND_B(NLD+1,N) ! STORE OF BANDED SYMMETRIC MATRIX "B".
                          ! "A(I,J)=BAND_A(M+J-I,I)", "B(I,J)=BAND_B(M+J-I,I)".
!-----------------------------------------------------------------------
INTEGER,PARAMETER:: FILE_66 = 66
REAL(RK),PARAMETER:: SMALL_A=1.E-4_RK, SMALL_B=1.E-4_RK
INTEGER M, I, J
INTEGER CNT
#define A(I,J)BAND_A(M+J-I,I)
#define B(I,J)BAND_B(M+J-I,I)
!-------------------------------------
  WRITE(*,       100) SMALL_A, SMALL_B
  WRITE(FILE_66, 100) SMALL_A, SMALL_B
  100 FORMAT('SET_BAND_AB_2: SMALL_A, SMALL_B=', ES12.5, ', ',ES12.5)
  M = NLD + 1
  CNT = 0
  DO  J = 1, N/2
    DO  I = J, MIN(J+NLD,N/2)
      A(I,J) = MAX(I,J)-1.0_RK
      B(I,J) = 1/(I+J-1.0_RK)
      CNT = CNT + 1
    ENDDO
    I = J
    B(I,J) = B(I,J)+1.0_RK
  ENDDO
  !---------------------------
  DO  J = MAX(1,(N/2+1)-NLD), (N/2)
    DO  I = (N/2+1), MIN(J+NLD,N)
      A(I,J) = SMALL_A
      B(I,J) = SMALL_B
      CNT = CNT + 1
    ENDDO
  ENDDO
  !---------------------------
  DO  J = (N/2+1), N
    DO  I = J, MIN(J+NLD,N)
      A(I,J) = MAX(I-N/2,J-N/2)-1.0_RK
      B(I,J) = 1/((I-N/2)+(J-N/2)-1.0_RK)
      CNT = CNT + 1
    ENDDO
    I = J
    B(I,J) = B(I,J)+1.0_RK
  ENDDO
  WRITE(*,       200) CNT, (1+NLD)*(2*N-NLD)/2
  WRITE(FILE_66, 200) CNT, (1+NLD)*(2*N-NLD)/2
  200 FORMAT('CNT=', I10, ', (1+NLD)*(2*N-NLD)/2=', I10)
  !---------------------------
#undef A
#undef B
END SUBROUTINE SET_BAND_AB_2
!==============================================================================
!
!   NPROB=3 : SET THE VALUES OF THE BANDED MATRICES "A" AND "B".
!   THE ELEMENTS OF MATRICES INSIDE THE BAND ARE GIVEN BY THE EXPRESSIONS:
!   "A_{P,Q} = P*Q/SQRT{P^2+Q^2}", "B_{P,Q}=1/(P+Q-1)+DELTA_{P,Q}",
!   WHERE "DELTA_{P,Q}" IS THE KRONECKER SYMBOL.
!
!==============================================================================
SUBROUTINE SET_BAND_AB_3 (N, NLD, BAND_A, BAND_B)
IMPLICIT NONE
INTEGER,PARAMETER:: RK = MY_RKIND
INTEGER, INTENT(IN) :: N               ! SIZE OF BOTH MATRICES "A" AND "B".
INTEGER, INTENT(IN) :: NLD             ! LOWER BANDWIDTH OF BOTH "A" AND "B".
REAL(RK),INTENT(OUT):: BAND_A(NLD+1,N) ! STORE OF BANDED SYMMETRIC MATRIX "A".
REAL(RK),INTENT(OUT):: BAND_B(NLD+1,N) ! STORE OF BANDED SYMMETRIC MATRIX "B".
                             ! A(I,J)=BAND_A(M+J-I,I), B(I,J)=BAND_B(M+J-I,I).
!-----------------------------------------------------------------------
INTEGER M, I, J
#define A(I,J)BAND_A(M+J-I,I)
#define B(I,J)BAND_B(M+J-I,I)
!-----------------------------
  M = NLD + 1
  DO  J = 1, N
    DO  I = J, MIN(J+NLD,N)
      A(I,J) = 1.0_RK*I*J/SQRT(REAL(I,KIND=RK)**2+REAL(J,KIND=RK)**2)
      B(I,J) = 1/(I+J-1.0_RK)
    ENDDO
    I = J
    B(I,J) = B(I,J)+1.0_RK
  ENDDO
#undef A
#undef B
END SUBROUTINE SET_BAND_AB_3
!==============================================================================
!
!  NPROB=4 : SET THE VALUES OF THE BANDED MATRICES "A" AND "B".
!  THE ELEMENTS OF MATRICES INSIDE THE BAND ARE GIVEN BY THE EXPRESSIONS:
!  "A_{P,Q}=MAX{P,Q}-1","B_{P,Q}=DELTA_{P,Q}+(1-DELTA_{P,Q})/(1+|P-Q|)/H", 
!  WHERE "H" IS THE LOWER BANDWIDTH AND "DELTA_{P,Q}" IS THE KRONECKER SYMBOL.
!
!==============================================================================
SUBROUTINE SET_BAND_AB_4 (N, NLD, BAND_A, BAND_B)
IMPLICIT NONE
INTEGER,PARAMETER:: RK = MY_RKIND
INTEGER, INTENT(IN) :: N               ! SIZE OF BOTH MATRICES "A" AND "B".
INTEGER, INTENT(IN) :: NLD             ! LOWER BANDWIDTH OF BOTH "A" AND "B".
REAL(RK),INTENT(OUT):: BAND_A(NLD+1,N) ! STORE OF BANDED SYMMETRIC MATRIX "A".
REAL(RK),INTENT(OUT):: BAND_B(NLD+1,N) ! STORE OF BANDED SYMMETRIC MATRIX "B".
                              ! A(I,J)=BAND_A(M+J-I,I), B(I,J)=BAND_B(M+J-I,I).
!-----------------------------------------------------------------------
INTEGER M, I, J
#define A(I,J)BAND_A(M+J-I,I)
#define B(I,J)BAND_B(M+J-I,I)
!-----------------------------
  M = NLD + 1
  DO  J = 1, N
    DO  I = J, MIN(J+NLD,N)
      A(I,J) = MAX(I,J)-1.0_RK
    ENDDO
  ENDDO
  DO  J = 1, N
    B(J,J) = 1.0_RK
    DO  I = J+1, MIN(J+NLD,N)
      B(I,J) = 1/(1.0_RK+ABS(I-J))/NLD
    ENDDO
  ENDDO
#undef A
#undef B
END SUBROUTINE SET_BAND_AB_4
