!******************************************************************************
!
! $Id: set_filter.F90,v 1.1 2010/07/10 01:44:28 murakami Exp $
!
!******************************************************************************
!==============================================================================
!
! THIS IS AN INTERFACE ROUTINE TO THE FILTER LIBRARY.
!   FOR THE SPECIFIED FILTER, FROM THE PARAMETERS "MU", "A_MAX", "A_MIN"
! OF THE FILTER, THE VALUE OF "N_MIN" IS CALCULATED WHICH IS MINIMAL VALUE 
! OF THE DEGREE "N" OF THE FILTER.
!
!==============================================================================
SUBROUTINE FILTER_GET_NMIN (FILT_TYPE, MU, A_MAX, A_MIN, N_MIN)
USE MOD_ELLPTC
IMPLICIT NONE
INTEGER,PARAMETER:: RK = MY_RKIND
INTEGER, INTENT(IN):: FILT_TYPE ! FILTER TYPE (1=BUTT,2=CHEB,3=ELLIP,4=INVCHEB).
REAL(RK),INTENT(IN):: MU           ! PARAMETER FOR TRANSITION REGION "MU".
REAL(RK),INTENT(IN):: A_MAX, A_MIN ! PARAMETERS FOR ATTENUATION "A_MAX","A_MIN".
INTEGER, INTENT(OUT):: N_MIN ! MINIMAL DEGREE OF FILTER TO SATISFY REQUIREMENTS.
!------------------------------------
REAL(RK) EPS
!------------------------------------
  SELECT CASE(FILT_TYPE)
    CASE(1); CALL FILTER_SETUP_BUTTER  (MU, A_MAX, A_MIN, EPS, N_MIN)
    CASE(2); CALL FILTER_SETUP_CHEB    (MU, A_MAX, A_MIN, EPS, N_MIN)
    CASE(3); CALL FILTER_SETUP_INVCHEB (MU, A_MAX, A_MIN, EPS, N_MIN)
    CASE(4); CALL FILTER_SETUP_ELLPTC  (MU, A_MAX, A_MIN, EPS, N_MIN)
  END SELECT
END SUBROUTINE FILTER_GET_NMIN
!==============================================================================
!
! THIS IS AN INTERFACE ROUTINE TO THE FILTER LIBRARY.
!  FOR THE SPECIFIED TYPE OF THE FILTER, THE POLES "ZPOLE"  (IN THE NORMAL
! COORDINATE) AND THEIR COEFFICIENTS "ZCOEF" AND ALSO "CINF" ARE CALCULATED 
! FROM THE DEGREE "N", TRANSITION PARAMETER "MU", AND "EPS" OF THE FILTER.
!
!==============================================================================
SUBROUTINE FILTER_POLES_COEFS (FILT_TYPE, N, MU, EPS, ZPOLE, ZCOEF, CINF) 
USE MOD_ELLPTC
IMPLICIT NONE
INTEGER,PARAMETER:: RK = MY_RKIND
INTEGER, INTENT(IN):: FILT_TYPE ! FILTER TYPE (1=BUTT,2=CHEB,3=ELLIP,4=INVCHEB).
INTEGER, INTENT(IN):: N   ! DEGREE OF THE FILTER "N".
REAL(RK),INTENT(IN):: MU  ! FILTER'S TRANSITION PARAMETER "MU".
REAL(RK),INTENT(IN):: EPS ! FILTER'S "EPS" VALUE.
COMPLEX(RK),INTENT(OUT):: ZPOLE(N) ! POLES WITH POSITIVE IMAGINARY PARTS.
COMPLEX(RK),INTENT(OUT):: ZCOEF(N) ! COEFFICIENTS OF POLES.
REAL(RK),   INTENT(OUT):: CINF     ! COEFFICIENTS OF CONSTANT TERM.
!---------------------------------------------------
INTEGER,PARAMETER:: FILE_66 = 66
REAL(RK) L, A_STOP
!---------------------------------------------------
  SELECT CASE(FILT_TYPE)
    CASE(1) ! BUTTERWORTH.
      CALL FILTER_ASTOP_BUTTER (EPS, MU, N, A_STOP)
      CALL FILTER_POLES_COEFS_BUTTER (N, EPS, ZPOLE, ZCOEF, CINF)
    CASE(2) ! CHEBYSHEV.
      CALL FILTER_ASTOP_CHEB (EPS, MU, N, A_STOP)
      CALL FILTER_POLES_COEFS_CHEB (N, EPS, ZPOLE, ZCOEF, CINF)
    CASE(3) ! INVERSE CHEBYSHEV.
      CALL FILTER_ASTOP_INVCHEB (EPS, MU, N, A_STOP)
      CALL FILTER_POLES_COEFS_INVCHEB(N, MU, EPS, ZPOLE, ZCOEF, CINF)
    CASE(4) ! ELLIPTIC.
      CALL FILTER_SETUP_ELLPTC2 (MU, N, L) ! TO MAKE THE VALUE OF "L".
      CALL FILTER_ASTOP_ELLPTC (EPS, L, A_STOP)
      CALL FILTER_POLES_COEFS_ELLPTC (N, MU, EPS, L, ZPOLE, ZCOEF, CINF)
  END SELECT
  !----------
  ! PRINT "A_STOP", THE ATTAINED MINIMAL VALUE OF ATTENUATION IN STOP BANDS.
  WRITE(*,      100) 10*LOG10(A_STOP)
  WRITE(FILE_66,100) 10*LOG10(A_STOP)
  100 FORMAT('A_STOP = ', F8.3,'[dB]')
  !--
END SUBROUTINE FILTER_POLES_COEFS
