!******************************************************************************
!
! $Id: set_job_params.F90,v 1.1 2010/07/10 01:44:28 murakami Exp $
!
!******************************************************************************
!==============================================================================
!
!   SET THE JOB PARAMETERS FROM THE INPUTS.
!
!==============================================================================
SUBROUTINE SET_JOB_PARAMS (NPROB, N, NLD, LAM0, LAM1, &
                           FILT_TYPE, MU, A_MAX, A_MIN, NF, &
                           NV, REL_THRES, &
                           MX_ITER)
IMPLICIT NONE
INTEGER,PARAMETER:: RK = MY_RKIND
INTEGER, INTENT(OUT):: NPROB ! THE TEST PROBLEM NUMBER.
INTEGER, INTENT(OUT):: N     ! THE PROBLEM MATRIX SIZE.
INTEGER, INTENT(OUT):: NLD   ! LOWER BANDWIDTH OF MATRICES "A" AND "B".
REAL(RK),INTENT(OUT):: LAM0, LAM1 ! BOTH ENDS OF THE INTERVAL.
INTEGER, INTENT(OUT):: FILT_TYPE  ! TYPE OF THE FILTER.
REAL(RK),INTENT(OUT):: MU, A_MAX, A_MIN ! PARAMETERS FOR THE FILTER.
INTEGER, INTENT(OUT):: NF        ! (HALF) DEGREE OF THE FILTER.
INTEGER, INTENT(OUT):: NV        ! NUMBER OF RANDOM VECTORS TO BE FILTERED.
REAL(RK),INTENT(OUT):: REL_THRES ! RELATIVE THRESHOLD FOR SVD.
INTEGER, INTENT(OUT):: MX_ITER   ! MAX NUMBER OF ITERATIONS FOR CORRECTIONS.
!-----------------------------------------------------------------------
INTEGER,PARAMETER:: FILE_66=66
!-------------------------------
  !===========================================
  PRINT *, 'INPUT (THE PROBLEM NUMBER) NPROB:'
  READ *, NPROB
  WRITE(      *,100) NPROB
  WRITE(FILE_66,100) NPROB
  100 FORMAT('NPROB = ', I3)
  !===========================================
  PRINT *, 'INPUT (THE SIZE OF MATRIX) N:'
  READ *, N
  PRINT *, 'INPUT (THE LOWER BAND WIDTH) NLD:'
  READ *, NLD
  WRITE(      *,200) N, NLD
  WRITE(FILE_66,200) N, NLD
  200 FORMAT('N = ', I8, ', NLD = ', I6)
  IF (N  <= 0) STOP 'ERROR: DETECTED N <= 0.'
  IF (NLD < 0) STOP 'ERROR: DETECTED NLD < 0.'
  !===========================================
  ! READ THE INTERVAL FOR EIGENVALUES TO BE SEARCHED.
  PRINT *, 'INPUT (THE INTERVAL OF EIGENVALUE): LAM0 AND LAM1.'
  READ *, LAM0, LAM1
  WRITE(      *, 300) LAM0, LAM1
  WRITE(FILE_66, 300) LAM0, LAM1
  300 FORMAT('THE INTERVAL: LAM0, LAM1 = ', G14.7, ',' , G14.7)
  IF (LAM0 >= LAM1) STOP 'ERROR: DETECTED LAM0 >= LAM1.'
  !===========================================
  ! READ THE FILTER PARAMETERS.
  CALL SET_FILTER_PARAMS (FILT_TYPE, MU, A_MAX, A_MIN, NF)
  !===========================================
  ! READ THE NUMBER OF RANDOM INITIAL VECTORS TO GENERATE.
  PRINT *, 'INPUT THE NUMBER OF RANDOM INITIAL VECTORS.'
  READ *, NV
  WRITE(      *, 700) NV
  WRITE(FILE_66, 700) NV
  700 FORMAT('NUMBER OF RANDOM INITIAL VECTORS: NV = ', I4)
  IF (NV <= 0) STOP 'ERROR: DETECTED NV <= 0.'
  !===========================================
  ! READ THE RELATIVE THRESHOLD FOR SVD.
  PRINT *, 'INPUT THE RELATIVE THRESHOLD FOR SVD.'
  READ *, REL_THRES
  WRITE(      *, 800) REL_THRES
  WRITE(FILE_66, 800) REL_THRES
  800 FORMAT('RELATIVE THRESHOLD: REL_THRES = ', ES9.3)
  IF (REL_THRES <= 0.0_RK) STOP 'ERROR: DETECTED REL_THRES <= 0.0.'
  !===========================================
  ! READ THE MAX NUMBER OF ITERATIONS FOR CORRECTION OF SOLUTIONS.
  PRINT *, 'INPUT THE MAX NUMBER OF ITERATIONS.'
  READ *, MX_ITER
  WRITE(      *, 900) MX_ITER
  WRITE(FILE_66, 900) MX_ITER
  900 FORMAT('MAX NUMBER OF ITERATIONS: MX_ITER = ', I2)
  IF (MX_ITER <0) STOP 'ERROR: DETECTED MX_ITER < 0.'
  !===========================================
END SUBROUTINE SET_JOB_PARAMS
!==============================================================================
!
! SET THE FILTER TYPE "FILT_TYPE" AND THE PARAMETERS "MU", "A_MAX", "A_MIN".
!
!==============================================================================
SUBROUTINE SET_FILTER_PARAMS (FILT_TYPE, MU, A_MAX, A_MIN, NF)
IMPLICIT NONE
INTEGER,PARAMETER:: RK = MY_RKIND
INTEGER, INTENT(OUT):: FILT_TYPE ! FILTER TYPE (1=BUTT,2=CHEB,3=ELLP,4=INVCHEB).
REAL(RK),INTENT(OUT):: MU, A_MAX, A_MIN ! THREE PARAMETERS OF THE FILTER.
INTEGER, INTENT(OUT):: NF               ! (HALF) DEGREE OF THE FILTER.
!------------------------------------------------------------------------------
INTEGER,PARAMETER:: FILE_66=66
CHARACTER(20) FILT_NAME
INTEGER N_MIN
!------------------------------
  !======================================
  ! READ THE TYPE OF THE FILTER TO CHOOSE.
  PRINT *, 'INPUT THE FILTER TYPE (1:BUTTER, 2:CHEB, 3:INV-CHEB, 4:ELLIPTIC).'
  READ *, FILT_TYPE
  SELECT CASE(FILT_TYPE)
    CASE(1); FILT_NAME = '(BUTTERWORTH)'
    CASE(2); FILT_NAME = '(CHEBYSHEV)'
    CASE(3); FILT_NAME = '(INVERSE CHEBYSHEV)'
    CASE(4); FILT_NAME = '(ELLIPTIC)'
    CASE DEFAULT; STOP 'ERROR: UNKNOWN FILTER TYPE.'
  END SELECT
  WRITE(      *, 100) FILT_TYPE, FILT_NAME
  WRITE(FILE_66, 100) FILT_TYPE, FILT_NAME
  100 FORMAT('FILTER TYPE = ', I1, 3X, A)
  !======================================
  ! READ THE FILTER PARAMETERS "A_MAX", "A_MIN".
  PRINT *, 'INPUT (THE FILTER PARAMETERS): A_MAX, A_MIN (BOTH IN [dB]).'
  READ *, A_MAX, A_MIN
  WRITE(      *, 200) A_MAX, A_MIN
  WRITE(FILE_66, 200) A_MAX, A_MIN
  200 FORMAT('FILTER PARAMETERS: A_MAX, A_MIN = ', &
               F10.5, '[dB] , ', F10.5, '[dB].')
  IF (A_MAX <= 0.0) STOP 'ERROR: DETECTED A_MAX <= 0.0.'
  IF (A_MIN <= 0.0) STOP 'ERROR: DETECTED A_MIN <= 0.0.'
  IF (A_MAX >= A_MIN) STOP 'ERROR: DETECTED A_MAX >= A_MIN.'
  A_MAX = 10.0_RK**(A_MAX/10.0_RK) ! TRANSFORM VALUE IN [dB] TO TRUE ONE.
  A_MIN = 10.0_RK**(A_MIN/10.0_RK) ! TRANSFORM VALUE IN [dB] TO TRUE ONE.
  !======================================
  ! READ THE FILTER PARAMETERS "MU".
  PRINT *, 'INPUT (THE FILTER PARAMETER): MU.'
  READ *, MU
  WRITE(      *, 300) MU
  WRITE(FILE_66, 300) MU
  300 FORMAT('FILTER PARAMETER: MU = ', ES13.6)
  IF (MU <= 1.0_RK) STOP 'ERROR: DETECTED MU <= 1.0.'
  !======================================
  ! CALCULATE "N_MIN" THE MINIMAL DEGREE OF THE FILTER.
  CALL FILTER_GET_NMIN (FILT_TYPE, MU, A_MAX, A_MIN, N_MIN)
  WRITE(      *, 400) N_MIN
  WRITE(FILE_66, 400) N_MIN
  400 FORMAT('FILTER N_MIN =', I6)
  !======================================
  ! INPUT THE VALUE OF "NF" THE DEGREE OF THE FILTER.
  PRINT *, 'INPUT THE (HALF) DEGREE OF THE FILTER: NF'
  READ *, NF
  IF (NF == 0) NF = N_MIN
  IF (NF < 0) THEN
    NF = ABS(NF)
    WRITE(      *, 510) NF
    WRITE(FILE_66, 510) NF
    510 FORMAT('FILTER DEGREE IS FORCED: NF = ', I6)
  ELSE
    WRITE(      *, 500) NF
    WRITE(FILE_66, 500) NF
    500 FORMAT('FILTER DEGREE: NF = ', I6)
    IF (NF < N_MIN) STOP 'ERROR: NF IS LESS THAN N_MIN.'
  ENDIF
  !======================================
END SUBROUTINE SET_FILTER_PARAMS
